import muon as mu
import pandas as pd 
import numpy as np
import argparse
import sys
import logging
import re
from muon import MuData
from anndata import AnnData


L = logging.getLogger()
L.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s: %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)
L.addHandler(log_handler)

# parse argumetns
parser = argparse.ArgumentParser()
parser.add_argument("--input_mudata",
                    default="mdata.h5mu",
                    help="file name, format: .h5mu")
parser.add_argument("--clusters_files_csv",
                    default=None,
                    help="comma separated table of mod,compiled clusters dataframes")
parser.add_argument("--umap_files_csv",
                    default=None,
                    help = "comma separated  table of mod,txt file containing umap coordinates")
parser.add_argument("--output_mudata",
                    default="mdata.h5mu",
                    help="file name, format: .h5mu")
args, opt = parser.parse_known_args()
L.info(args)
mdata = mu.read(args.input_mudata)

L.info("loading clusters")
cf = pd.read_csv(args.clusters_files_csv)

if isinstance(mdata, MuData):
    pass
elif isinstance(mdata, AnnData):
    mds = cf['mod'].unique().tolist()
    if len(mds)>1:
        sys.exit("You have clustered multiple modalities but are providing only a unimodal anndata")
    else:
        L.warn("found one modality, converting to mudata: %s " % mds[0] )    
        tmp = MuData({mds[0]:mdata})
        del mdata
        mdata = tmp
        del tmp

# add in the clusters



for i in range(cf.shape[0]):
    cf_df = pd.read_csv(cf['fpath'][i], sep='\t', index_col=0) 
    cf_df['clusters'] = cf_df['clusters'].astype('str').astype('category')
    cf_df = cf_df.rename(columns={"clusters":cf['new_key'][i]})
    
    if cf['mod'][i] != "multimodal":
        mdata[cf['mod'][i]].obs = mdata[cf['mod'][i]].obs.merge(cf_df, left_index=True, right_index=True)
    else:
        mdata.obs = mdata.obs.merge(cf_df, left_index=True, right_index=True)

uf = pd.read_csv(args.umap_files_csv)


for i in range(uf.shape[0]):
    uf_df = pd.read_csv(uf['fpath'][i], sep='\t', index_col=0) 
    mod = uf['mod'][i]
    new_key = uf['new_key'][i]
    if uf['mod'][i] != "multimodal":
        if all(mdata[mod].obs_names == uf_df.index):
            mdata[mod].obsm[new_key] =  uf_df.to_numpy()
        else:
            L.warn("cannot integrate %s into mdata as obs_names mismatch" % uf.iloc[i,:] )
    else:
        # check the observations are the same
        if set(mdata.obs_names).difference(uf_df.index) == set():
            # put the observations in the same order
            uf_df = uf_df.loc[mdata.obs_names,:]
            mdata.obsm[new_key] =  uf_df.to_numpy()
        else:
            L.warning("cannot integrate %s into mdata as obs_names mismatch" % uf.iloc[i,:] )

mdata.write(args.output_mudata)
mdata.obs.to_csv(re.sub(".h5mu", "_cell_metdata.tsv", args.output_mudata), sep='\t')

L.info("done")



import muon as mu
import scib
import pandas as pd
import numpy as np
import seaborn as sns
import scanpy as sc
sc.set_figure_params(figsize=(8, 6), dpi=300)
import argparse
import gc
import os
import sys
import logging
L = logging.getLogger()
L.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s: %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)
L.addHandler(log_handler)

# load arguments 
parser = argparse.ArgumentParser()
parser.add_argument('--query',
                    default='',
                    help='the query anndata/mudata')
parser.add_argument('--repuse',
                    default=None,
                    help='which latent representation to use to calculate the metrics')
parser.add_argument('--covariate',
                    default=None,
                    help='the convariate in the query anndata/mudata you want to use to calculate the metrics')
parser.add_argument('--batch_key',
                    default=None,
                    help='the convariate in the query anndata/mudata you want to use to calculate the metrics')
parser.add_argument('--cluster_key',
                    default=None,
                    help='string of column in query anndata/mudata containing cluster assignments you want to use to calculate the metrics (ARI/NMI)')
parser.add_argument('--outdir', default=None,
                    help='directory to save the data to')

args, opt = parser.parse_known_args()

L.info(args)


if args.covariate is not None:
    covariates_use = args.covariate.split(",")
    covariates_use = [a.strip() for a in covariates_use]
else:
    sys.exit("i don't have covariates to calculate metrics on")


repuse = str(args.repuse)

mdata = mu.read(args.query)
if type(mdata) is mu.MuData:
    if "rna" not in mdata.mod.keys():
        sys.exit("we only support querying using RNA but your mdata doesn't contain rna")
    else:
        input_adata = mdata["rna"].copy()
del mdata


adata_query = input_adata[input_adata.obs['is_reference'] == 'Query'].copy()
L.info("repuse is %s" %(repuse))
if repuse not in adata_query.obsm.keys():
    sys.exit("the latent representation is not in the obsm.keys of this query")

L.info("query is:")
print(adata_query)

L.info("Calculating scib metrics using ground truth covariates:")
print(covariates_use)

if args.cluster_key is None:
    cluster_key = "leiden_" + repuse 
    L.info("you didn't specify cluster_key, so i'm using %s " % cluster_key)
else:
    cluster_key = str(args.cluster_key)
    L.info("cluster_key used is: %s" %cluster_key)

results = {}
for labelk in covariates_use:
    m={"ASW_scaled":scib.metrics.silhouette(adata_query, labelk, repuse, metric='euclidean', scale=True),
        "ARI":scib.metrics.ari(adata_query, label_key=labelk, cluster_key=cluster_key), #predictions or leiden clustering
        "NMI":scib.metrics.nmi(adata_query, label_key=labelk, cluster_key=cluster_key),
        "graph_connectivity":scib.metrics.graph_connectivity(adata_query, labelk)}    
    if args.batch_key is not None:
        m["clisi_graph_embed"]= scib.me.clisi_graph(adata_query, label_key=labelk, type_="knn", batch_key=str(args.batch_key))

    results[labelk] = m

file_name= os.path.splitext(os.path.basename(args.query).replace("query_to_reference_", "").replace(".h5mu", ""))[0]
#"query_to_reference_" + model_name + "_" + latent_choice + ".h5mu"
L.info("saving file output")
file_out = os.path.join(args.outdir,("scib.query_"+ file_name+".csv"))
print(file_out)
#save file to txt
pres = pd.DataFrame.from_dict(results,orient='index')
pres.to_csv(file_out, sep="\t")
L.info("Finished scib")


import pytest
# from scpipelines.funcs.processing import extract_parameter_from_fname
import panpipes.funcs as pnp
from anndata import AnnData
from muon import MuData
import pandas as pd
import numpy as np

@pytest.fixture()
def mudata():
    yield MuData(
        {
            "mod1": AnnData(np.arange(0, 100, 0.1).reshape(-1, 10)),
            "mod2": AnnData(np.arange(101, 2101, 1).reshape(-1, 20)),
        }
    )

@pytest.fixture()
def anndata():
    yield AnnData(np.arange(0, 50, 1).reshape(-1, 10), 
        obs=pd.DataFrame(index=[f"cell{i}" for i in range(5)],
                         data={'sample_id': ['a', 'a', 'b', 'b', 'c']}),
        var=pd.DataFrame(index=[f"gene{i}" for i in range(10)]))



@pytest.fixture()
def anndata_with_obs():
    yield AnnData(np.arange(0, 50, 1).reshape(-1, 10), 
        obs=pd.DataFrame(index=[f"cell{i}" for i in range(5)],
                         data={'sample_id': ['a', 'a', 'b', 'b', 'c'],
                               'batch': [1,1,2,2,3]}),
        var=pd.DataFrame(index=[f"gene{i}" for i in range(10)])
    )

@pytest.fixture()
def anndata_with_var():
    yield AnnData(np.arange(0, 50, 1).reshape(-1, 10), 
        obs=pd.DataFrame(index=[f"cell{i}" for i in range(5)],
                         data={'sample_id': ['a', 'a', 'b', 'b', 'c']}),
        var=pd.DataFrame(index=[f"gene{i}" for i in range(10)],
                         data={'feature_type': "Gene Expression",
                              'new_index': [f"newgene{i}" for i in range(10)]}
                        )
    )


def test_extract_parameter_from_fname():
    assert pnp.pp.extract_parameter_from_fname("test_res0.6_cluster.txt.gz", "res", "test") == 0.6
    assert pnp.pp.extract_parameter_from_fname("test_res1.0_cluster.txt.gz", "res", "test") == 1
    assert pnp.pp.extract_parameter_from_fname("test_methodeuclidean_cluster.txt.gz", "method", "test") == "euclidean"
    assert pnp.pp.extract_parameter_from_fname("method_methodeuclidean_cluster.txt.gz", "method", "method") == "euclidean"

def test_is_float_try():
    assert pnp.pp.is_float_try("1") is True
    assert pnp.pp.is_float_try("cheese") is False

def test_splitall():
    assert pnp.pp.splitall("path") == ['path']
    assert pnp.pp.splitall("path/to/cheese") == ['path', 'to', 'cheese']
    assert pnp.pp.splitall("/path/to/cheese") == ['path', 'to', 'cheese']

def test_test_file_or_value():
    assert pnp.pp.test_file_or_value(4) == "value"
    assert pnp.pp.test_file_or_value(__file__) == "file"
    with pytest.raises(ValueError):
        pnp.pp.test_file_or_value("cheese")

def test_which_ind():
    assert pnp.pp.which_ind([True, True, False]) == [0,1]
    assert pnp.pp.which_val([True, True, False], [1,2,3]) == [1,2]

def test_check_for_bool():
    assert pnp.pp.check_for_bool('True') is True
    assert pnp.pp.check_for_bool(True) is True
    assert pnp.pp.check_for_bool('False') is False
    assert pnp.pp.check_for_bool(False) is False
    with pytest.raises(TypeError):
        pnp.pp.check_for_bool('cheese')
    with pytest.raises(TypeError):
        pnp.pp.check_for_bool(4)

def test_intersection():
    assert pnp.pp.intersection(['a', 'b', 'c'], ['c', 'd', 'e']) == ['c']


def assert_match_anndata(ad1, ad2):
    assert np.array_equal(ad1.X, ad2.X)
    assert all(ad1.obs == ad2.obs)
    assert all(ad1.var == ad2.var)

def test_concat_adatas(anndata):
    ad1 = anndata
    ad1 = ad1.copy()
    ad1.obs['sample_id'] = 'a'
    ad2 = anndata
    ad2 = ad2.copy()
    ad2.obs['sample_id'] = 'b'
    # this is what the merged version should look like :-) 
    adata_full = AnnData(
        np.concatenate([np.arange(0, 50, 1),np.arange(0, 50, 1)]).reshape(-1, 10),
                obs = pd.DataFrame(index=['cell0-a', 'cell1-a', 'cell2-a', 'cell3-a', 'cell4-a', 
                                            'cell0-b', 'cell1-b','cell2-b', 'cell3-b', 'cell4-b'],
                                    data={'sample_id': ['a', 'a', 'a', 'a', 'a', 
                                                        'b', 'b', 'b', 'b', 'b']}, dtype="category"),
                var=pd.DataFrame(index=[f"gene{i}" for i in range(10)])
    )
    merg_data = pnp.pp.concat_adatas([ad1, ad2],
        batch_key="sample_id", 
        batch_categories=['a', 'b'], 
        join_type="inner")
    # testing the case where there are two anndatas to be merged
    assert np.array_equal(merg_data.X, adata_full.X)
    assert_match_anndata(merg_data, adata_full)
    # testing the case where there is only 1 anndata sent to function
    assert_match_anndata(pnp.pp.concat_adatas([ad1], batch_key="sample_id", 
        batch_categories=['a']), ad1)
    # test the corner case where an anddata is passed
    with pytest.raises(TypeError):
        pnp.pp.concat_adatas(ad1, batch_key="sample_id", 
        batch_categories=['a'])

def test_merge_with_adata_obs(anndata, anndata_with_obs):
    new_obs = pd.DataFrame(data={'sample_id': ['a', 'b', 'c'],
                             'batch': [1, 2, 3]})
    # test exceptions
    with pytest.raises(TypeError):
        pnp.pp.merge_with_adata_obs("cheese", new_obs, "sample_id")
    with pytest.raises(TypeError):
        pnp.pp.merge_with_adata_obs(anndata, "cheese", "sample_id")
    with pytest.raises(KeyError):
        pnp.pp.merge_with_adata_obs(anndata, new_obs, "cheese")
    with pytest.raises(KeyError):
        pnp.pp.merge_with_adata_obs(anndata, new_obs, "batch")
    anndata.obs = pnp.pp.merge_with_adata_obs(anndata, new_obs, "sample_id")
    assert_match_anndata(anndata, anndata_with_obs)


def test_merge_with_adata_obs_inplace(anndata, anndata_with_obs):
    new_obs = pd.DataFrame(data={'sample_id': ['a', 'b', 'c'],
                             'batch': [1, 2, 3]})
    pnp.pp.merge_with_adata_obs(anndata, new_obs, "sample_id", inplace=True)
    assert_match_anndata(anndata, anndata_with_obs)

def test_remove_unused_categories():
    df = pd.DataFrame(data={"a1":['a', 'b', 'c'], 'a2':['c', 'd', 'e']}, dtype='category')
    df = df.iloc[0:2,:]
    pnp.pp.remove_unused_categories(df)
    assert df['a1'].cat.categories.tolist() == ['a', 'b']
    assert df['a2'].cat.categories.tolist() == ['c', 'd']
    with pytest.raises(TypeError):
        pnp.pp.remove_unused_categories("cheese")

def test_add_var_mtd(anndata, anndata_with_var):
    new_var = pd.DataFrame(index=[f"gene{i}" for i in range(10)],
                         data={'feature_type': "Gene Expression",
                                'new_index': [f"newgene{i}" for i in range(10)]}
                        )
    pnp.pp.add_var_mtd(anndata, new_var, left_on="index", right_on="new_index")
    assert_match_anndata(anndata, anndata_with_var)


def test_add_var_mtd_types(anndata):
    with pytest.raises(TypeError):
        pnp.pp.add_var_mtd(anndata, 'cheese')
    new_var = pd.DataFrame(index=[f"gene{i}" for i in range(10)],
                         data={'feature_type': "Gene Expression",
                                'new_index': [f"newgene{i}" for i in range(10)]}
                        )
    with pytest.raises(TypeError):
        pnp.pp.add_var_mtd('cheese', new_var)
    # check mismatch var df raises coorect errors


# def test_add_var_mtd_warnings(anndata, anndata_with_var):
#     ad = anndata.copy()
#     new_var = pd.DataFrame(index=[f"gene{i}" for i in range(9)],
#                          data={'feature_type': "Gene Expression",
#                                 'new_index': [f"newgene{i}" for i in range(9)]})
#     # with pytest.warns(UserWarning):
#     #     pnp.pp.add_var_mtd(ad, new_var)


def test_update_var_index(anndata):
    new_index = [f"newgene{i}" for i in range(10)]
    anndata.var['new_index'] = new_index
    pnp.pp.update_var_index(anndata, "new_index")
    assert all(anndata.var.index == new_index)





'''
scanpy QC spatial
'''
import sys
import logging
L = logging.getLogger()
L.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s: %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)
L.addHandler(log_handler)

import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)

import pandas as pd
import os
import argparse
import scanpy as sc
import muon as mu

from panpipes.funcs.io import write_obs


parser = argparse.ArgumentParser()
# required option
parser.add_argument("--input_anndata",
                    default="adata_raw.h5ad",
                    help="")
parser.add_argument("--spatial_filetype",
                    default="",
                    help="")
parser.add_argument("--outfile",
                    default="adata_unfilt.h5ad",
                    help="")
parser.add_argument("--figdir",
                    default="./figures/",
                    help="path to save the figures to")
parser.add_argument("--ccgenes",
                    default=None,
                    help="path to file containing cell cycle genes")
parser.add_argument("--customgenesfile",
                    default=None,
                    help="path to file containing list of genes to quantify")
parser.add_argument("--calc_proportions",
                    default=None,
                    help="which list of genes to use to calc proportion of mapped reads over total,per cell?")
parser.add_argument("--score_genes",
                    default=None,
                    help="which list of genes to use to scanpy.tl.score_genes per cell?")

args, opt = parser.parse_known_args()

L.info("running scanpy spatial qc pipeline")

sc.settings.verbosity = 3


L.info("running with args:")
L.info(args)
figdir = args.figdir

if not os.path.exists(figdir):
    os.mkdir(figdir)

sc.settings.figdir = figdir
sc.set_figure_params(scanpy=True, fontsize=14, dpi=300, facecolor='white', figsize=(5,5))


L.info("reading in data")

mdata = mu.read(args.input_anndata)
spatial = mdata['spatial']

L.info("spatial data is")
print(spatial)
L.info("sample id")
print(spatial.obs["sample_id"])

qc_vars = []

if args.customgenesfile is not None:
    if os.path.exists(args.customgenesfile):
        cat_dic = {}
        customgenes = pd.read_csv(args.customgenesfile)
        custom_cat = list(set(customgenes['group'].tolist()))
        for cc in custom_cat:
            cat_dic[cc] = customgenes.loc[customgenes["group"] == cc,"feature"].tolist()
            
        # define qc_vars 
        if args.calc_proportions is not None:
            calc_proportions = args.calc_proportions.split(",")
            calc_proportions = [a.strip() for a in calc_proportions]
            for kk in calc_proportions:
                xname= kk
                gene_list = cat_dic[kk]
                spatial.var[xname] = [x in gene_list for x in spatial.var_names] 
                qc_vars.append(xname)
        else:
            L.info("no genes to calc proportions for")
           
        # Score genes 
        if args.score_genes is not None:
            score_genes = args.score_genes.split(",")
            score_genes = [a.strip() for a in score_genes]
            for kk in score_genes:
                xname= kk
                gene_list = cat_dic[kk]
                sc.tl.score_genes(spatial, gene_list , 
                                    ctrl_size=min(len(gene_list), 50), 
                                    gene_pool=None, 
                                    n_bins=25, 
                                    score_name=kk + '_score', 
                                    random_state=0, 
                                    copy=False, 
                                    use_raw=None)
        else:
            L.info("no genes to calc scores for")    
        
    else:
        sys.exit("the path of the custom genes file does not exist")

        
        
        
# Calculate QC metrics 
L.info("calculating QC metrics")
percent_top = [50, 100, 200, 500] #default
percent_top = [x for x in percent_top if x <= spatial.n_vars]
sc.pp.calculate_qc_metrics(spatial, qc_vars=qc_vars, percent_top=percent_top, inplace=True)

if args.spatial_filetype == "vizgen":
    spatial.obsm["blank_genes"].to_numpy().sum() / spatial.var["total_counts"].sum() * 100

# Calculate cc scores 
if args.ccgenes is not None:
    ccgenes = pd.read_csv(args.ccgenes, sep='\t')
    sgenes = ccgenes[ccgenes["cc_phase"] == "s"]["gene_name"].tolist()
    g2mgenes = ccgenes[ccgenes["cc_phase"] == "g2m"]["gene_name"].tolist()
    L.info("calculating CC scores")
    sc.tl.score_genes_cell_cycle(spatial, s_genes=sgenes, g2m_genes=g2mgenes)

# Aug 2023: we now need to update the mdata object to pick the calc proportion outputs made on 
# spatial = mdata['spatial']
print(spatial.obs.columns)
print(mdata.obs.columns)

mdata.update()

single_id = os.path.basename(str(args.input_anndata))
single_id = single_id.replace("_raw.h5mu","")
L.info("updated metadata")
print(mdata.obs.columns)

L.info("saving obs in a metadata tsv file")
write_obs(mdata, output_prefix=single_id, output_suffix="_cell_metadata.tsv")
L.info("saving mudata")
mdata.write(args.outfile)

L.info("done")




import argparse
import os
import sys
import logging
import scanpy as sc
from muon import MuData, read
from panpipes.funcs.scmethods import run_neighbors_method_choice
from panpipes.funcs.io import read_yaml
from panpipes.funcs.scmethods import lsi

L = logging.getLogger()
L.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s: %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)
L.addHandler(log_handler)


parser = argparse.ArgumentParser()
parser.add_argument('--infile', default='./anndata.h5mu',
                    help="file name, format: anndata_scaled.h5ad")
parser.add_argument('--outfile', default='./anndata_wneighbors.h5ad',
                    help="file name, format: .h5ad")
parser.add_argument('--neighbor_dict', default=None,
                    help="helps find the correct dimension reduction for sc.pp.neighbors(), default='X_pca'")
parser.add_argument('--n_threads', default=1,help='number of threads available')

args, opt = parser.parse_known_args()
L.info(args)
# load the filtering dictionary 
neighbor_dict = args.neighbor_dict
if isinstance(args.neighbor_dict, dict):
    neighbor_dict = args.neighbor_dict
else:
    neighbor_dict = read_yaml(args.neighbor_dict) 

sc.settings.n_jobs = int(args.n_threads)
L.info("Running with options: %s", args)

# read data
L.info("reading mudata")
mdata = read(args.infile)



for mod in neighbor_dict.keys():
    if neighbor_dict[mod]['use_existing']:
        L.info('using existing neighbors graph for %s' % mod)
        pass
    else:
        L.info("computing new neighbors for %s" % mod)
        if type(mdata) is MuData:
            adata=mdata[mod]
        if (neighbor_dict[mod]['dim_red'] == "X_pca") and ("X_pca" not in adata.obsm.keys()):
            L.info("X_pca not found, computing it using default parameters")
            sc.tl.pca(adata)
            if (mod == "atac") and (neighbor_dict[mod]['dim_remove'] is not None):
                dimrem = int(neighbor_dict[mod]['dim_remove'])
                adata.obsm['X_pca'] = adata.obsm['X_pca'][:, dimrem:]
                adata.varm["PCs"] = adata.varm["PCs"][:, dimrem:]
        if mod == "atac":
            if (neighbor_dict[mod]['dim_red'] == "X_lsi") and ("X_lsi" not in adata.obsm.keys()):
                L.info("X_lsi not found, computing it using default parameters")
                lsi(adata=adata, num_components=50)
                if neighbor_dict[mod]['dim_remove'] is not None:
                    dimrem = int(neighbor_dict[mod]['dim_remove'])
                    adata.obsm['X_lsi'] = adata.obsm['X_lsi'][:, dimrem:]
                    adata.varm["LSI"] = adata.varm["LSI"][:, dimrem:]
                    adata.uns["lsi"]["stdev"] = adata.uns["lsi"]["stdev"][dimrem:]

        # run command
        opts = dict(method=neighbor_dict[mod]['method'],
                    n_neighbors=int(neighbor_dict[mod]['k']),
                    n_pcs=int(neighbor_dict[mod]['n_dim_red']),
                    metric=neighbor_dict[mod]['metric'],
                    nthreads=args.n_threads,
                    use_rep=neighbor_dict[mod]['dim_red'])


        run_neighbors_method_choice(adata,**opts)
        mdata.update()



L.info("saving data")
mdata.write(args.outfile)
L.info("done")


"""
Scanpy inbuilt plots for custom markers
CRG 2020-06-19
"""

import scanpy as sc
from muon import read, MuData
sc.settings.autoshow = False
import pandas as pd
import argparse
from anndata import AnnData
import matplotlib
matplotlib.use('agg')
import os 
import re

import sys
import logging
L = logging.getLogger()
L.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s: %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)
L.addHandler(log_handler)

# parse argumetns
parser = argparse.ArgumentParser()
parser.add_argument("--infile",
                    default="mdata.h5mu",
                    help="file name, format: .h5mu")
parser.add_argument("--modalities",
                    default="rna",
                    help="list of modalities to search for UMAPs in")
args, opt = parser.parse_known_args()

L.info("running with:")
L.info(args)
# args = argparse.Namespace(infile='mdata_clustered.h5mu', figdir=None)
# ---------

def main(adata,figdir):
    # get all possible umap coords
    pattern="X_umap(.*)"
    obsm_keys = [x for x in adata.obsm.keys() if re.search(pattern, x)]
    L.info("Umap keys founds %s" % obsm_keys)
    # get all possible clustersclusters
    pattern=re.compile(r'^leiden|^louvain') 
    cluster_keys  = [x for x in adata.obs.columns if re.search(pattern, x)]
    L.info("Cluster keys founds %s" % cluster_keys)
    if len(obsm_keys) == 0  or len(cluster_keys) == 0:
        return
    
    # make sure clusters are categories
    for ck in cluster_keys:
        adata.obs[ck] = adata.obs[ck].astype('category')
    # plot all the umaps
    for ok in obsm_keys:
        fig = sc.pl.embedding(adata, basis = ok,color=cluster_keys,
         show=False, return_fig=True, legend_loc='on data')
        for ax in fig.axes:
            ax.set(xlabel="UMAP_1", ylabel="UMAP_2")
        fig.suptitle(ok, y=1.0)
        fig.savefig(os.path.join(figdir, ok +  "_clusters.png"))

def plot_spatial(adata,figdir):
    # get all possible umap coords
    pattern="spatial(.*)"
    obsm_keys = [x for x in adata.obsm.keys() if re.search(pattern, x)]
    L.info("Umap keys founds %s" % obsm_keys)
    # get all possible clustersclusters
    pattern=re.compile(r'^leiden|^louvain') 
    cluster_keys  = [x for x in adata.obs.columns if re.search(pattern, x)]
    L.info("Cluster keys founds %s" % cluster_keys)
    if len(obsm_keys) == 0  or len(cluster_keys) == 0:
        return
    
    # make sure clusters are categories
    for ck in cluster_keys:
        adata.obs[ck] = adata.obs[ck].astype('category')
    # plot all the umaps
    for ok in obsm_keys:
        fig = sc.pl.embedding(adata, basis = ok,color=cluster_keys,
         show=False, return_fig=True, legend_loc='on data')
        for ax in fig.axes:
            ax.set(xlabel="spatial1", ylabel="spatial2")
        fig.suptitle(ok, y=1.0)
        fig.savefig(os.path.join(figdir, ok +  "_clusters.png"))


L.debug("load data")
mdata = read(args.infile)

mods = args.modalities.split(',')
# detemin initial figure directory based on object type

# do plotting
if 'multimodal' in mods:
    if os.path.exists("multimodal/figures") is False:
        os.makedirs("multimodal/figures")
    main(mdata, figdir="multimodal/figures")


# we also need to plot per modality
if type(mdata) is MuData:
    for mod in mdata.mod.keys():
        if mod in mods:
            L.info("plotting for modality: %s" % mod)
            figdir  = os.path.join(mod, "figures")
            if os.path.exists(figdir) is False:
                os.makedirs(figdir)
            if mod == "spatial": # added separate function for spatial
                plot_spatial(mdata[mod], figdir)
            else:
                main(mdata[mod], figdir)





L.info('done')


from ruffus import *
import sys
import os
from cgatcore import pipeline as P
import pandas as pd
import cgatcore.iotools as IOTools
import re
from itertools import chain
import glob
import warnings
import logging
from panpipes.funcs.io import dictionary_stripper
# from itertools import chain
# import glob

# import pandas as pd

PARAMS = P.get_parameters(
    ["%s/pipeline.yml" % os.path.splitext(__file__)[0],
     "pipeline.yml"])


PARAMS['py_path'] =  os.path.join(os.path.dirname(os.path.dirname(__file__)), 'python_scripts')
PARAMS['r_path'] = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'R_scripts')
PARAMS['resources_path'] = os.path.join(os.path.dirname(os.path.dirname(__file__)), "resources")

if  PARAMS['sample_prefix'] is not None:
    PARAMS['mudata_file'] = PARAMS['sample_prefix'] + ".h5mu"
else:
    logging.warning("sample prefix is None, please use a string")
    
job_kwargs = {}

if PARAMS['condaenv'] is not None:
    job_kwargs["job_condaenv"] =PARAMS['condaenv']


mode_dictionary = PARAMS["modalities"]
@mkdir("logs")
@mkdir("figures")
@originate(PARAMS['mudata_file'])
def filter_mudata(outfile):
    if PARAMS['filtering_run']:
        filter_dict = dictionary_stripper(PARAMS['filtering'])
        cmd = """
        python %(py_path)s/run_filter.py
        --input_mudata %(unfiltered_obj)s
        --output_mudata %(outfile)s
        --filter_dict "%(filter_dict)s"
        """
        if PARAMS['filtering_keep_barcodes'] is not None:
            cmd += " --keep_barcodes %(filtering_keep_barcodes)s"
        if PARAMS['intersect_mods'] is not None:
            cmd += " --intersect_mods %(intersect_mods)s"
        cmd += " > logs/filtering.log "
        job_kwargs["job_threads"] = PARAMS['resources_threads_low']
        P.run(cmd, **job_kwargs)
    else:
        try:
            f = open(outfile)
            f.close(outfile)
        except IOError:
            print("filter is not set to True, but there is no %s file" % outfile)
            sys.exit(1)


def run_plotqc_query(pqc_dict):
    # avoid deleting from PARAMS
    pqc = pqc_dict.copy()
    del pqc['grouping_var']
    return any([x != None for x in pqc.values()])


@active_if(PARAMS['filtering_run'])
@active_if(run_plotqc_query(PARAMS['plotqc']))
@follows(filter_mudata)
@originate("logs/postfilterplot.log")
def postfilterplot(log_file):
    cell_mtd_file = PARAMS['sample_prefix'] + "_filtered_cell_metadata.tsv"
    cmd = """
    Rscript %(r_path)s/plotQC.R 
    --prefilter FALSE
    --cell_metadata %(cell_mtd_file)s 
    --sampleprefix %(sample_prefix)s
    --groupingvar %(plotqc_grouping_var)s
    """
    cmd += " --scanpy_or_muon muon"
    if PARAMS['plotqc']['rna_metrics'] is not None:
        cmd += " --rna_qc_metrics %(plotqc_rna_metrics)s"
    if PARAMS['plotqc']['prot_metrics'] is not None:
        cmd += " --prot_qc_metrics %(plotqc_prot_metrics)s"
    if PARAMS['plotqc']['atac_metrics'] is not None:
        cmd += " --atac_qc_metrics %(plotqc_atac_metrics)s"
    if PARAMS['plotqc']['rep_metrics'] is not None:
        cmd += " --rep_qc_metrics %(plotqc_rep_metrics)s"
    cmd += " > %(log_file)s "
    job_kwargs["job_threads"] = PARAMS['resources_threads_low']
    P.run(cmd, **job_kwargs)




@active_if(PARAMS['downsample_n'] is not None)
@follows(filter_mudata)
@originate("logs/downsample.log", PARAMS['mudata_file'])
def downsample(log_file, filt_obj):
    if PARAMS['downsample'] is not None:
        cmd="""
        python %(py_path)s/downsample.py
         --input_mudata %(filt_obj)s \
         --output_mudata %(filt_obj)s \
         --sampleprefix %(sample_prefix)s \
         --downsample_value %(downsample_n)s \
         --downsample_col %(downsample_col)s
         --intersect_mods %(downsample_mods)s
        """
        cmd += " > %(log_file)s  "
        job_kwargs["job_threads"] = PARAMS['resources_threads_low']
        P.run(cmd, **job_kwargs)
    IOTools.touch_file(log_file)
    pass


# setting these follows means that it still works if filter is False
@active_if(mode_dictionary['rna'] is True)
@follows(downsample)
@follows(mkdir('figures/rna'))
@transform(filter_mudata, formatter(), "logs/preprocess_rna.log")
def rna_preprocess(adata_obj, log_file):
    cmd = """python %(py_path)s/run_preprocess_rna.py 
            --input_mudata %(adata_obj)s 
            --fig_dir figures/rna/ 
            --output_scaled_mudata %(adata_obj)s
            """
    # add in the options if specified in pipeline.yml
    if PARAMS['hvg_exclude'] is not None:
        cmd += " --exclude_file %(hvg_exclude_file)s"
        if PARAMS['hvg_exclude'] =="default":
            cmd += " --exclude exclude"
        else:    
            cmd += " --exclude %(hvg_exclude)s"
    if PARAMS['hvg_flavor'] is not None:
        cmd += " --flavor %(hvg_flavor)s"
    if PARAMS['hvg_batch_key'] is not None:
        cmd += " --hvg_batch_key %(hvg_batch_key)s"
    if PARAMS['hvg_n_top_genes'] is not None:
        cmd += " --n_top_genes %(hvg_n_top_genes)s"
    if PARAMS['hvg_min_mean'] is not None:
        cmd += " --min_mean %(hvg_min_mean)s"
    if PARAMS['hvg_max_mean'] is not None:
        cmd += " --max_mean %(hvg_max_mean)s"
    if PARAMS['hvg_min_disp'] is not None:
        cmd += " --min_disp %(hvg_min_disp)s"
    if PARAMS['hvg_filter'] is not None:
        cmd += " --filter_by_hvg %(hvg_filter)s"
    if PARAMS['regress_variables'] is not None:
        cmd += " --regress_out %(regress_variables)s"
    if PARAMS['run_scale'] is True:
        cmd += " --scale True"
        if PARAMS['scale_max_value'] is not None:
            cmd += " --scale_max_value %(scale_max_value)s"
    else:
        cmd += " --scale False"
    if PARAMS['pca_scree_n_pcs'] is not None:
        cmd += " --n_pcs %(pca_n_pcs)s"
    if PARAMS['pca_solver'] is not None:
        if PARAMS['pca_solver'] == "default":
            cmd += " --pca_solver arpack"
        else:
            cmd += " --pca_solver %(pca_solver)s"
    if PARAMS['pca_color_by'] is not None:
        cmd += " --color_by %(pca_color_by)s"
    cmd += " > %(log_file)s "
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)


@active_if(mode_dictionary['prot'] is True)
@follows(rna_preprocess)
@originate("logs/preprocess_prot.log", PARAMS['mudata_file'])
def prot_preprocess( log_file, scaled_file, ):
    if os.path.exists("figures/prot") is False:
        os.mkdir("figures/prot" )
    cmd = """
        python %(py_path)s/run_preprocess_prot.py
        --filtered_mudata %(scaled_file)s
        --figpath ./figures/prot
        --save_mudata_path %(scaled_file)s
        """
    if PARAMS['normalisation_methods'] is not None:
        cmd += " --normalisation_methods %(prot_normalisation_methods)s"
    if PARAMS['quantile_clipping'] is not None:
        cmd += " --quantile_clipping %(prot_quantile_clipping)s"
    if PARAMS['clr_margin'] is not None:
        cmd += " --clr_margin %(prot_clr_margin)s"
    if PARAMS['background_obj'] is not None:
        cmd += " --bg_mudata %(prot_background_obj)s"
    if PARAMS['prot']['store_as_X'] is not None:
        cmd += " --store_as_x %(prot_store_as_X)s"
    if PARAMS['prot_save_norm_prot_mtx'] is True:
        cmd += " --save_mtx True"
    if PARAMS["prot_pca"] is True:
        if PARAMS['prot_solver'] is not None:
            if PARAMS['prot_solver'] == "default":
                cmd += " --pca_solver arpack"
            else:
                cmd += " --pca_solver %(pca_solver)s"
        cmd += """ 
        --run_pca True
        --n_pcs %(prot_n_pcs)s
        --color_by %(prot_color_by)s
        """
    cmd += " > %(log_file)s"
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)


@active_if(mode_dictionary['atac'] is True)
@follows(prot_preprocess)
# @transform(rna_preprocess,formatter(),"logs/preprocess_atac.log")
@originate("logs/preprocess_atac.log", PARAMS['mudata_file'])
def atac_preprocess(log_file, scaled_file):
    if os.path.exists("figures/atac") is False:
        os.mkdir("figures/atac" )
    cmd = """
        python %(py_path)s/run_preprocess_atac.py
        --input_mudata %(scaled_file)s
        --output_mudata %(scaled_file)s
        --figdir ./figures/atac
        """
    if PARAMS['atac_binarize'] is True:
        cmd += " --binarize True" 
    else:
        cmd += " --binarize False" 
    if PARAMS['atac_normalize'] is not None:
        cmd += " --normalize %(atac_normalize)s"
    if PARAMS['atac_TFIDF_flavour'] is not None:
        cmd += " --TFIDF_flavour %(atac_TFIDF_flavour)s"
    if PARAMS['atac_min_mean'] is not None:
        cmd += " --min_mean %(atac_min_mean)s"    
    if PARAMS['atac_max_mean'] is not None:
        cmd += " --max_mean %(atac_max_mean)s"    
    if PARAMS['atac_min_disp'] is not None:
        cmd += " --min_disp %(atac_min_disp)s"    
    if PARAMS['atac_n_top_features'] is not None:
        cmd += " --n_top_features %(atac_n_top_features)s" 
    if PARAMS["atac_filter_by_hvf"] is not None:
        if PARAMS["atac_filter_by_hvf"] is True:
            cmd += " --filter_by_hvf True"
        else: 
            cmd += " --filter_by_hvf False"
    if PARAMS['atac_dimred'] is not None:
        cmd += " --dimred %(atac_dimred)s"
    else:    
        cmd += " --dimred PCA"
    if PARAMS['atac_n_comps'] is not None:
        cmd += " --n_comps %(atac_n_comps)s"
    if PARAMS['atac_solver'] is not None:
        if PARAMS['atac_solver'] == "default":
            cmd += " --solver arpack"
        else:    
            cmd += " --solver %(atac_solver)s"
    if PARAMS['atac_dim_remove'] is not None:
        cmd += " --dim_remove %(atac_dim_remove)s"
    if PARAMS['atac_feature_selection_flavour'] is not None:
        cmd += " --feature_selection_flavour %(atac_feature_selection_flavour)s"
    if PARAMS['atac_min_cutoff'] is not None:
        cmd += " --min_cutoff %(atac_min_cutoff)s"
    cmd += " --color_by %(atac_color_by)s" 
    cmd += " > %(log_file)s"
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)



# @active_if(mode_dictionary['rep'] is True)
# @follows(atac_preprocess)
# # @transform(rna_preprocess,formatter(),"logs/preprocess_rep.log")
# @originate("logs/preprocess_rep.log", PARAMS['mudata_file'])
# def rep_preprocess( log_file, scaled_file):
#     pass

# ---- end stub
@follows(postfilterplot,rna_preprocess,atac_preprocess,prot_preprocess)
def full():
    """
    All cgat pipelines should end with a full() function which updates,
    if needed, all branches of the pipeline.
    The @follows statement should ensure that all functions are covered,
    either directly or as prerequisites.
    """
    pass


def main(argv=None):
    if argv is None:
        argv = sys.argv
    P.main(argv)


if __name__ == "__main__":
    sys.exit(P.main(sys.argv))



import argparse
import yaml
# import scanpy as sc
import pandas as pd
# import numpy as np
# from scipy.sparse import csr_matrix
import muon as mu
import warnings
from muon._atac.tools import add_peak_annotation, locate_fragments
import squidpy as sq
from mudata import MuData
import os
"""
this script copies the make_adata_from_csv.py that creates
ONE MUDATA PER SAMPLE, with in each ONE LAYER per modality
for cell-suspension, saves them to temp. 
concatenation of the mudatas saved in tmp happens 
in the concat_anndata.py script
"""

import sys
import logging

L = logging.getLogger()
L.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s: %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)
L.addHandler(log_handler)

from panpipes.funcs.io import check_filetype, read_yaml


parser = argparse.ArgumentParser()
#parser.add_argument('--mode_dictionary',default=mode_dictionary,type=json.loads)
parser.add_argument('--mode_dictionary',default="",
                    help="accepts a yaml speficication of modalities dictionary", 
                    type=yaml.safe_load)
parser.add_argument('--sample_id',
                    default=None,
                    help='')
parser.add_argument('--output_file',
                    default=None,
                    help='')
# start of spatial args
parser.add_argument('--spatial_infile', 
                    default=None,
                    help='')
parser.add_argument('--spatial_filetype', 
                    default=None,
                    help='')
parser.add_argument('--spatial_counts', 
                    default=None,
                    help='')
parser.add_argument('--spatial_metadata', 
                    default=None,
                    help='')
parser.add_argument('--spatial_transformation', 
                    default=None,
                    help='')

parser.set_defaults(verbose=True)
args, opt = parser.parse_known_args()
L.info(args)

L.info("starting the code now")
# unimodal mu (check if all the modalities)
if isinstance(args.mode_dictionary, dict):
    mode_dictionary = args.mode_dictionary
else:
    mode_dictionary = read_yaml(args.mode_dictionary) 
#{'spatialT': True}

permf = [key for key, value in mode_dictionary.items() if value == True]
all_files = {
            "spatial":[args.spatial_infile, #path, mandatory for squidpy
                        args.spatial_filetype, #needed for the load_adata_in function to call one of vizgen,visium
                        args.spatial_counts, #name of the counts file, mandatory for squidpy
                        args.spatial_metadata, #name of the metadata file, mandatory for squidpy
                        args.spatial_transformation]}
#subset to the modalities we want from permf (in this case only spatial)
all_files = {nm: x  for (nm, x) in all_files.items() if nm in permf}

#[check_filetype(x[0], x[1]) for x in all_files.values()]
# read the spatial data with one of the functions inside
# load_mdata_from_multiple_files
#     |
#      -------->load_adata_in
# this function creates ONE mudata per row of the CAF file 
# and saves it with sample_id.h5mu in tmp/ 


def create_soft_link(source_path, target_path):
    try:
        os.symlink(source_path, target_path)
        print(f"Soft link created: {target_path}")
    except FileExistsError:
        print(f"Soft link already exists: {target_path}")
    except Exception as e:
        print(f"An error occurred while creating soft link: {e}")

def check_dir_transform(infile_path, transform_file):
    images_path = os.path.join(infile_path, "images")
    if not os.path.exists(images_path):
        os.makedirs(images_path)
        print(f"Directory '{images_path}' created.")
    target_path = os.path.join(images_path, os.path.basename(transform_file))
    tocheck_orig= os.path.join(infile_path,transform_file)
    if os.path.exists(target_path):
        print("File already exists in 'images' directory.")
    elif os.path.islink(tocheck_orig):
        original_path = os.path.realpath(tocheck_orig)
        create_soft_link(original_path, target_path)
    else:
        create_soft_link(os.path.abspath(transform_file), target_path)



if args.spatial_filetype=="vizgen":
    adata = sq.read.vizgen(path = args.spatial_infile, #path, mandatory for squidpy
                        counts_file=args.spatial_counts, #name of the counts file, mandatory for squidpy
                        meta_file = args.spatial_metadata, #name of the metadata file, mandatory for squidpy
                        transformation_file=args.spatial_transformation,
                        library_id = str(args.sample_id)) #this also has kwargs for read_10x_h5 but keep simple
    adata.uns["spatial"][str(args.sample_id)]["scalefactors"]["transformation_matrix"].columns = adata.uns["spatial"][str(args.sample_id)]["scalefactors"]["transformation_matrix"].columns.astype(str)
elif args.spatial_filetype =="visium":
    adata = sq.read.visium(path = args.spatial_infile, #path, mandatory for squidpy
                        counts_file=args.spatial_counts, #name of the counts file, mandatory for squidpy
                        library_id = str(args.sample_id)
                        ) #this also has kwargs for read_10x_h5 but keep simple

L.info("adata is now: %s" % adata)
L.info("creating mudata")

mdata = MuData({"spatial": adata})


#---------------
# do some extra processing on the different modalities as needed
#---------------

#make var names unique
for mm in mdata.mod.keys():
    mdata[mm].var_names_make_unique()

mdata.obs['sample_id'] = str(args.sample_id)

# copy the sample_id to each modality
for mm in mdata.mod.keys():
    print("saving sample_id to each modality")
    # mdata[mm].obs['sample_id'] = mdata.obs['sample_id']
    mdata[mm].obs['sample_id'] = mdata.obs.loc[mdata[mm].obs_names,:]['sample_id']

mdata.update()


L.info("saving data")
L.debug(mdata)
# this will write the file as anndata or muon depending on the class of mdata.
mdata.write(args.output_file)

L.info("done")





from venv import create
from ruffus import *
import sys
import os
from cgatcore import pipeline as P
import pandas as pd
import cgatcore.iotools as IOTools
import re
from itertools import chain
import glob


PARAMS = P.get_parameters(
    ["%s/pipeline.yml" % os.path.splitext(__file__)[0],
     "pipeline.yml"])


PARAMS['py_path'] =  os.path.join(os.path.dirname(os.path.dirname(__file__)), 'python_scripts')
PARAMS['r_path'] = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'R_scripts')
PARAMS['resources_path'] = os.path.join(os.path.dirname(os.path.dirname(__file__)), "resources")

sprefix = PARAMS['sample_prefix']

job_kwargs={}
if PARAMS['condaenv'] is not None:
    job_kwargs["job_condaenv"] =PARAMS['condaenv']


preprocessed_file = PARAMS['preprocessed_file']

@originate("logs/setup_dirs.sentinel")
def set_up_dirs(log_file):
    os.makedirs("logs", exist_ok=True)
    os.makedirs("tmp", exist_ok=True)
    os.makedirs("batch_correction", exist_ok=True)
    if PARAMS['rna']['run']:
        os.makedirs("figures/rna")
    if PARAMS['prot']['run']:
        os.makedirs("figures/prot")
    if PARAMS['atac']['run']:
        os.makedirs("figures/atac")
    if PARAMS['multimodal']['run']:
        os.makedirs("figures/multimodal")
    os.makedirs("figures/rep")
    IOTools.touch_file(log_file)
    pass
# ------------------------------------------------------------------------
# Unimodal Integration
# ------------------------------------------------------------------------
#
#
# ------------------------------------------------------------------------
# Rna unimodal integration:
# - nobatch
# - BBKNN
# - COMBAT
# - Harmony
# - Scanorama
# - scvi
# ------------------------------------------------------------------------
#rna No batch correction
@follows(set_up_dirs)
#@active_if(PARAMS['rna_run']) #TO DO enable this
@originate("batch_correction/umap_rna_none.csv")
def run_no_batch_umap(outfile):
    # print(PARAMS)
    rna_params = PARAMS['rna']

    cmd = """python %(py_path)s/batch_correct_none.py 
     --input_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s
     --integration_col %(rna_column)s
     """
    # cannot use the normal method for importing params from yaml, because it only works up to depth 2
    neighbor_params = PARAMS['rna']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    cmd += " > logs/rna_no_correct.log"
    
    if PARAMS['queues_long'] is not None:
        job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']    
    P.run(cmd, **job_kwargs) 



# rna BBKNN
@follows(set_up_dirs)
@active_if(PARAMS['rna_run'])
@active_if(PARAMS['rna_tools'] is not None and 'bbknn' in PARAMS['rna_tools'])
# @follows(normalise_log_hvg_regress_scale)
@originate("batch_correction/umap_rna_bbknn.csv")
def run_bbknn_rna(outfile):
    cmd = """python %(py_path)s/batch_correct_bbknn.py 
     --input_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s
     --integration_col %(rna_column)s
     --modality rna
     """
    if PARAMS['rna']['bbknn']['neighbors_within_batch'] is not None:
        cmd += " --neighbors_within_batch %i" % PARAMS['rna']['bbknn']['neighbors_within_batch']
    if PARAMS['rna']['neighbors']['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s" % PARAMS['rna']['neighbors']['npcs']
    cmd += " > logs/rna_bbknn.log "
    
    if PARAMS['queues_long'] is not None:
        job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs) 


# rna COMBAT
@follows(set_up_dirs)
@active_if(PARAMS['rna_run'])
@active_if(PARAMS['rna_tools'] is not None and 'combat' in PARAMS['rna_tools'])
@originate("batch_correction/umap_rna_combat.csv")
def run_combat(outfile):
    cmd = """python %(py_path)s/batch_correct_combat.py 
     --input_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s
     --integration_col %(rna_column)s
     --n_threads %(resources_threads_high)s
     --modality rna
     """
    # cannot use the normal method for importing params from yaml, because it only works up to depth 2
    neighbor_params = PARAMS['rna']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    cmd += " > logs/rna_combat.log "
    
    if PARAMS['queues_long'] is not None:
        job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']

    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs) 

# rna HARMONY
@follows(set_up_dirs)
@active_if(PARAMS['rna_run'])
@active_if(PARAMS['rna_tools'] is not None and 'harmony' in PARAMS['rna_tools'])
@originate("batch_correction/umap_rna_harmony.csv")
def run_harmony(outfile):
    cmd = """python %(py_path)s/batch_correct_harmony.py 
     --input_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s 
     --integration_col %(rna_column)s
     --n_threads %(resources_threads_high)s
     --modality rna
     """
    # cannot use the normal method for importing params from yaml, because it only works up to depth 2
    harmony_params = PARAMS['rna']['harmony']
    if harmony_params['npcs'] is not None:
        cmd += " --harmony_npcs %s" % harmony_params['npcs']
    if harmony_params['sigma'] is not None:
        cmd += " --sigma_val %s" % harmony_params['sigma'] 
    if harmony_params['theta'] is not None:
        cmd += " --theta_val %s" % harmony_params['theta']       
    neighbor_params = PARAMS['rna']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    cmd += " > logs/rna_harmony.log " 
     #job arguments
    
    if PARAMS['queues_long'] is not None:
        job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)

# rna SCANORAMA
@follows(set_up_dirs)
@active_if(PARAMS['rna_run'])
@active_if(PARAMS['rna_tools'] is not None and 'scanorama' in PARAMS['rna_tools'])
@originate("batch_correction/umap_rna_scanorama.csv")
def run_scanorama(outfile):
    cmd = """python %(py_path)s/batch_correct_scanorama.py 
     --input_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s 
     --integration_col %(rna_column)s
     --n_threads %(resources_threads_high)s
     --modality rna
     """
    # cannot use the normal method for importing params from yaml, because it only works up to depth 2
    neighbor_params = PARAMS['rna']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    cmd += " > logs/rna_scanorama.log " 
    #job arguments
    
    if PARAMS['queues_long'] is not None:
        job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)

# rna scvi
@follows(set_up_dirs)
@active_if(PARAMS['rna_run'])
@active_if(PARAMS['rna_tools'] is not None and 'scvi' in PARAMS['rna_tools'])
@originate("batch_correction/umap_rna_scvi.csv")
def run_scvi(outfile):
    cmd = """python %(py_path)s/batch_correct_scvi.py 
     --scaled_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s 
     --integration_col %(rna_column)s
     --figdir figures/rna/
     """
    # cannot use the normal method for importing params from yaml, because it only works up to depth 2
    neighbor_params = PARAMS['rna']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    cmd += " > logs/rna_scvi.log "
    job_kwargs = {}
    if PARAMS['queues_gpu'] is not None:
        job_kwargs["job_queue"] = PARAMS['queues_gpu']
        job_kwargs["job_threads"] = int(PARAMS['resources_threads_gpu'])
    elif PARAMS['queues_long'] is not None:
            job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
            job_kwargs["job_threads"] = int(PARAMS['resources_threads_high'])
    else:
        job_kwargs["job_threads"] = int(PARAMS['resources_threads_high'])
    P.run(cmd, **job_kwargs)

# rna scib
# @transform([run_scvi, run_scanorama, run_bbknn, run_harmony, run_combat],
#             regex(r"batch_correction/umap_rna_(.*).csv"),
#             r'batch_correction/scib_metrics_\1.csv',r'\1', 
#             scaled_file)
# def run_scib(infiles, outfile, method,  uncorrected_anndata):
#     print(infiles, outfile)
#     log_file = "logs/rna_scib_" + method + ".log"
#     # this is going to be apain since I want the temp h5ad files not the csvs
#     # get batch method from infile
#     corrected_anndata = "tmp/" + method + "_scaled_adata.h5ad"
#     cmd = """python %(py_path)s/run_scib_metrics.py 
#     --uncorrected_anndata %(uncorrected_anndata)s \
#     --batch_corrected_anndata %(corrected_anndata)s \
#     --integration_method %(method)s \
#     --integration_col %(rna_column)s \
#     --outfile %(outfile)s  
#     """
#     if PARAMS['use_muon']:
#         cmd += " --use_muon True"
#     if PARAMS['reference_col'] is not None:
#         cmd += " --reference_col %(scib_reference_col)s "
#     if PARAMS['scib_rough_clust'] is not None:
#         cmd += " --rough_consensus_clustering  %(scib_rough_clust)s"

#     cmd += " > %(log_file)s "
#     print(cmd)
#     P.run(cmd, job_threads=PARAMS['resources_threads_high'])
@active_if(PARAMS['rna_run'])
@follows(run_harmony, run_bbknn_rna, run_combat, run_scanorama, run_scvi, run_no_batch_umap)
def run_unimodal_integration_rna():
    pass

# ------------------------------------------------------------------------
# Prot unimodal Integration:
# - nobatch
# - Harmony
# ------------------------------------------------------------------------

@follows(set_up_dirs)
@active_if(PARAMS['prot_run'])
@originate("batch_correction/umap_prot_none.csv")
def run_no_batch_umap_prot(outfile):
    cmd = """python %(py_path)s/batch_correct_none.py 
     --input_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s
     --integration_col %(prot_column)s
     """
    cmd += " --modality prot"
    # cannot use the normal method for importing params from yaml, because it only works up to depth 2
    neighbor_params = PARAMS['prot']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    cmd += " > logs/prot_no_correct.log"
    
    if PARAMS['queues_long'] is not None:
        job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs) 

# prot HARMONY
@follows(set_up_dirs)
@active_if(PARAMS['prot_run'])
@active_if(PARAMS['prot_tools'] is not None and 'harmony' in PARAMS['prot_tools'])
@originate("batch_correction/umap_prot_harmony.csv")
def run_harmony_prot( outfile):
    cmd = """python %(py_path)s/batch_correct_harmony.py 
     --input_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s 
     --integration_col %(prot_column)s
     --n_threads %(resources_threads_high)s
     """
    cmd += " --modality prot"
    harmony_params = PARAMS['prot']['harmony']
    if harmony_params['npcs'] is not None:
        cmd += " --harmony_npcs %s" % harmony_params['npcs']
    if harmony_params['sigma'] is not None:
        cmd += " --sigma_val %s" % harmony_params['sigma'] 
    if harmony_params['theta'] is not None:
        cmd += " --theta_val %s" % harmony_params['theta']   
    neighbor_params = PARAMS['prot']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    cmd += " > logs/prot_harmony.log " 
     #job arguments
    
    if PARAMS['queues_long'] is not None:
        job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)



# prot BBKNN
@follows(set_up_dirs)
@active_if(PARAMS['prot_run'])
@active_if(PARAMS['prot_tools'] is not None and 'bbknn' in PARAMS['prot_tools'])
# @follows(normalise_log_hvg_regress_scale)
@originate("batch_correction/umap_prot_bbknn.csv")
def run_bbknn_prot(outfile):
    cmd = """python %(py_path)s/batch_correct_bbknn.py 
     --input_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s
     --integration_col %(prot_column)s
     --modality prot
     """
    if PARAMS['prot']['bbknn']['neighbors_within_batch'] is not None:
        cmd += " --neighbors_within_batch %i" % PARAMS['prot']['bbknn']['neighbors_within_batch']
    if PARAMS['prot']['neighbors']['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s" % PARAMS['prot']['neighbors']['npcs']
    cmd += " > logs/prot_bbknn.log "
    
    if PARAMS['queues_long'] is not None:
        job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs) 


# prot COMBAT
@follows(set_up_dirs)
@active_if(PARAMS['prot_run'])
@active_if(PARAMS['prot_tools'] is not None and 'combat' in PARAMS['prot_tools'])
@originate("batch_correction/umap_prot_combat.csv")
def run_combat_prot(outfile):
    cmd = """python %(py_path)s/batch_correct_combat.py 
     --input_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s
     --integration_col %(prot_column)s
     --n_threads %(resources_threads_high)s
     --modality prot
     """
    # cannot use the normal method for importing params from yaml, because it only works up to depth 2
    neighbor_params = PARAMS['prot']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    cmd += " > logs/prot_combat.log "
    
    if PARAMS['queues_long'] is not None:
        job_kwargs["job_queue"] = PARAMS['queues_long']

    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs) 
    

@active_if(PARAMS['prot_run'])
@follows(run_harmony_prot, run_bbknn_prot,run_combat_prot, run_no_batch_umap_prot)
def run_unimodal_integration_prot():
    pass


# ------------------------------------------------------------------------
# ATAC unimodal Integration:
# - nobatch
# - Harmony
# - BBKNN
# ------------------------------------------------------------------------
@follows(set_up_dirs)
@active_if(PARAMS['atac_run'])
@originate("batch_correction/umap_atac_none.csv")
def run_no_batch_umap_atac(outfile):
    cmd = """python %(py_path)s/batch_correct_none.py 
     --input_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s
     --integration_col %(atac_column)s
     """
    cmd += " --modality atac"
    neighbor_params = PARAMS['atac']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    atac_dimred = PARAMS['atac']['dimred']
    if PARAMS['atac']['dimred'] is not None:
        cmd += " --dimred %s" % atac_dimred
    else:
        cmd += " --dimred PCA"
    cmd += " > logs/atac_no_correct.log"
    
    if PARAMS['queues_long'] is not None:
        job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs) 

# atac HARMONY
@follows(set_up_dirs)
@active_if(PARAMS['atac_run'])
@active_if(PARAMS['atac_tools'] is not None and 'harmony' in PARAMS['atac_tools'])
@originate("batch_correction/umap_atac_harmony.csv")
def run_harmony_atac( outfile):
    cmd = """python %(py_path)s/batch_correct_harmony.py 
     --input_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s 
     --integration_col %(atac_column)s
     --n_threads %(resources_threads_high)s
     --modality atac
     """

    harmony_params = PARAMS['atac']['harmony']
    if harmony_params['npcs'] is not None:
        cmd += " --harmony_npcs %s" % harmony_params['npcs']
    if harmony_params['sigma'] is not None:
        cmd += " --sigma_val %s" % harmony_params['sigma'] 
    if harmony_params['theta'] is not None:
        cmd += " --theta_val %s" % harmony_params['theta']   
    neighbor_params = PARAMS['atac']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    atac_dimred = PARAMS['atac']['dimred']
    if PARAMS['atac']['dimred'] is not None:
        cmd += " --dimred %s" % atac_dimred
    else:
        cmd += " --dimred PCA"
    cmd += " > logs/atac_harmony.log " 
     #job arguments
    
    if PARAMS['queues_long'] is not None:
        job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)

# atac BBKNN
@follows(set_up_dirs)
@active_if(PARAMS['atac_run'] )
@active_if(PARAMS['atac_tools'] is not None and 'bbknn' in PARAMS['atac_tools'])
# @follows(normalise_log_hvg_regress_scale)
@originate("batch_correction/umap_atac_bbknn.csv")
def run_bbknn_atac(outfile):
    cmd = """python %(py_path)s/batch_correct_bbknn.py 
     --input_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s
     --integration_col %(atac_column)s
     --modality atac
     """
    if PARAMS['atac']['bbknn']['neighbors_within_batch'] is not None:
        cmd += " --neighbors_within_batch %i" % PARAMS['atac']['bbknn']['neighbors_within_batch']
    if PARAMS['atac']['neighbors']['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s" % PARAMS['atac']['neighbors']['npcs']
    #Forcing bbknn to run on PCA in case of atac
    cmd += " --dimred PCA"
    cmd += " > logs/atac_bbknn.log "
    if PARAMS['queues_long'] is not None:
        job_kwargs["job_queue"] = PARAMS['queues_long']
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs) 


@active_if(PARAMS['atac_run'])
@follows(run_harmony_atac, run_no_batch_umap_atac, run_bbknn_atac)
def run_unimodal_integration_atac():
    pass

@follows(run_unimodal_integration_rna,
         run_unimodal_integration_prot,
         run_unimodal_integration_atac)
def run_unimodal_integration():
    pass
# ------------------------------------------------------------------------
# Multimodal  Integration
# ------------------------------------------------------------------------
# RUN totalvi
@follows(set_up_dirs)
@active_if(PARAMS['multimodal_run'])
@active_if(PARAMS['multimodal_tools'] is not None)
@active_if('totalvi' in [x.lower() for x in PARAMS['multimodal_tools']])
@originate("batch_correction/umap_multimodal_totalvi.csv")
def run_totalvi(outfile):
    if os.path.exists("figures/multimodal") is False:
        os.makedirs("figures/multimodal")
    cmd = """python %(py_path)s/batch_correct_totalvi.py 
     --scaled_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s 
     --figdir figures/
     """
    if PARAMS['multimodal_column_categorical'] is not None:
        cmd += "--integration_col_categorical %(multimodal_column_categorical)s "
    neighbor_params = PARAMS['multimodal']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    cmd += " > logs/multimodal_totalvi.log "
    
    if PARAMS['queues_gpu'] is not None:
        job_kwargs["job_queue"] = PARAMS['queues_gpu']
        job_kwargs["job_threads"] = int(PARAMS['resources_threads_gpu'])
    else:
        if PARAMS['queues_long'] is not None:
            job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
        job_kwargs["job_threads"] = int(PARAMS['resources_threads_high'])
    P.run(cmd, **job_kwargs)

# Run MultiVI

@follows(set_up_dirs)
@active_if(PARAMS['multimodal_run'])
@active_if( PARAMS['multimodal_tools'] is not None)
@active_if('multivi' in [x.lower() for x in PARAMS['multimodal_tools']])
@originate("batch_correction/umap_multimodal_multivi.csv")
def run_multivi(outfile):
    if os.path.exists("figures/multimodal") is False:
        os.makedirs("figures/multimodal")
    cmd = """python %(py_path)s/batch_correct_multivi.py 
     --scaled_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s 
     --figdir figures/
     """
    
    if PARAMS['multimodal_column_categorical'] is not None:
        cmd += "--integration_col_categorical %(multimodal_column_categorical)s "

    if PARAMS['multimodal_column_continuous'] is not None:
        cmd += "--integration_col_continuous %(multimodal_column_continuous)s "

    neighbor_params = PARAMS['multimodal']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    cmd += " > logs/multimodal_multivi.log "
    
    if PARAMS['queues_gpu'] is not None:
        job_kwargs["job_queue"] = PARAMS['queues_gpu']
        job_kwargs["job_threads"] = int(PARAMS['resources_threads_gpu'])
    else:
        if PARAMS['queues_long'] is not None:
            job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
        job_kwargs["job_threads"] = int(PARAMS['resources_threads_high'])
    P.run(cmd, **job_kwargs)



# Run Mofa
@follows(set_up_dirs)
@active_if(PARAMS['multimodal_run'])
@active_if( PARAMS['multimodal_tools'] is not None and 'mofa' in PARAMS['multimodal_tools'])
@originate("batch_correction/umap_multimodal_mofa.csv")
def run_mofa(outfile):
    if os.path.exists("figures/multimodal") is False:
        os.makedirs("figures/multimodal")
    cmd = """python %(py_path)s/batch_correct_mofa.py 
     --scaled_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s 
     --figdir figures/
     """
    if PARAMS['multimodal_column_categorical'] is not None:
        cmd += " --integration_col_categorical %(multimodal_column_categorical)s"
    mofa_params = PARAMS['multimodal']['mofa']
    if mofa_params['n_factors'] is not None:
        cmd += " --n_factors %s" % mofa_params['n_factors']
    if mofa_params['n_iterations'] is not None:
        cmd += " --n_iterations %s" % mofa_params['n_iterations']
    if mofa_params['convergence_mode'] is not None:
        cmd += " --convergence_mode %s" % mofa_params['convergence_mode']
    if mofa_params['save_parameters']:
        cmd += " --save_parameters %s" % mofa_params['save_parameters']
        cmd += " --outfile_model %s" % mofa_params['outfile']
    neighbor_params = PARAMS['multimodal']['neighbors']
    if neighbor_params['method'] is not None:
        cmd += " --neighbors_method %s" % neighbor_params['method']
    if neighbor_params['metric'] is not None:
        cmd += " --neighbors_metric %s" % neighbor_params['metric']
    if neighbor_params['npcs'] is not None:
        cmd += " --neighbors_n_pcs %s"  % neighbor_params['npcs']
    if neighbor_params['k'] is not None:
        cmd += " --neighbors_k %s" % neighbor_params['k']
    
    if PARAMS['queues_gpu'] is not None:
        cmd += " --use_gpu True"
        job_kwargs["job_queue"] = PARAMS['queues_gpu']
        job_kwargs["job_threads"] = int(PARAMS['resources_threads_gpu'])
    else:
        if PARAMS['queues_long'] is not None:
            job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
        job_kwargs["job_threads"] = int(PARAMS['resources_threads_high'])
    cmd += " > logs/multimodal_mofa.log "
    P.run(cmd, **job_kwargs)

# Run WNN

@follows(set_up_dirs)
@follows(run_unimodal_integration)
@active_if(PARAMS['multimodal_run'])
@active_if( PARAMS['multimodal_tools'] is not None)
@active_if('wnn' in [x.lower() for x in PARAMS['multimodal_tools']])
@originate("batch_correction/umap_multimodal_wnn.csv")
def run_wnn(outfile):
    if os.path.exists("figures/multimodal") is False:
        os.makedirs("figures/multimodal")
    cmd = """python %(py_path)s/batch_correct_wnn.py 
     --scaled_anndata %(preprocessed_obj)s
     --output_csv %(outfile)s 
     --figdir figures/
     """
    wnn_params = PARAMS['multimodal']['WNN']
    if wnn_params['n_neighbors'] is not None:
       cmd += " --n_neighbors %s" % wnn_params['n_neighbors']
    if wnn_params['n_bandwidth_neighbors'] is not None:
       cmd += " --n_bandwidth_neighbors %s" % wnn_params['n_bandwidth_neighbors']
    if wnn_params['n_multineighbors'] is not None:
       cmd += " --n_multineighbors %s" % wnn_params['n_multineighbors']
    if wnn_params['metric'] is not None:
       cmd += " --metric %s" % wnn_params['metric']
    if wnn_params['low_memory'] is not None:
       cmd += " --low_memory %s" % wnn_params['low_memory']
    cmd += " > logs/multimodal_wnn.log"

    
    if PARAMS['queues_gpu'] is not None:
        job_kwargs["job_queue"] = PARAMS['queues_gpu']
        job_kwargs["job_threads"] = int(PARAMS['resources_threads_gpu'])
    else:
        if PARAMS['queues_long'] is not None:
            job_kwargs["job_queue"] = job_queue=PARAMS['queues_long']
        job_kwargs["job_threads"] = int(PARAMS['resources_threads_high'])
        
    P.run(cmd, **job_kwargs)

# end of multimodal
@follows(run_mofa, run_wnn, run_totalvi, run_multivi)
def run_multimodal_integration():
    pass
# ------------------------------------------------------------------------
# Evaluation
# ------------------------------------------------------------------------

@collate([run_no_batch_umap, run_scanorama, run_bbknn_rna,run_harmony, run_scvi, run_combat, 
          run_no_batch_umap_prot, run_harmony_prot, run_bbknn_prot, run_combat_prot,
          run_no_batch_umap_atac, run_bbknn_atac,run_harmony_atac, 
          run_totalvi, run_wnn, run_multivi, run_mofa], 
         regex(r"(.*)/(.*)"), 
          r'batch_correction/combined_umaps.tsv')
def collate_integration_outputs(infiles,outfile):
    #infiles_string = ','.join(infiles)
    contents = glob.glob("batch_correction/*.csv")
    union = list(set(infiles) | set(contents))
    infiles_string = ','.join(union)
    batch_dict =  "batch_correction/batch_dict.yml"
    cell_mtd_file = sprefix + "_cell_mtd.csv"
    cmd = """python %(py_path)s/run_collate_mtd_files.py 
    --input_mudata %(preprocessed_obj)s
    --input_umap_files %(infiles_string)s 
    --output_batch_yml %(batch_dict)s 
    --output_cell_metadata_csv %(cell_mtd_file)s
    --output_combined_umaps_tsv %(outfile)s
    """ 
        # add the integration cols
    if PARAMS['rna_run']:
        cmd += " --rna_integration_col %(rna_column)s"
    if PARAMS['prot_run']:
        cmd += " --prot_integration_col %(prot_column)s"
    if PARAMS['atac_run']:
        cmd += " --atac_integration_col %(atac_column)s"
    if PARAMS['multimodal_run']:
        cmd += " --multimodal_integration_col %(multimodal_column_categorical)s"
    cmd += " > logs/collate_mtd.log "
    job_kwargs["job_threads"] = PARAMS['resources_threads_medium']
    P.run(cmd, **job_kwargs)  


# this COLLATE job will become the big final collate job across all possible combinations you may have run
@follows(set_up_dirs)
@transform(collate_integration_outputs, formatter(), "logs/plot_batch_corrected_umaps.log")
def plot_umaps(infile, outfile):
    print(infile, outfile)
    cell_mtd_file = sprefix + "_cell_mtd.csv"
    cmd = """python %(py_path)s/plot_umaps_batch_correct.py 
    --fig_dir figures/
    --combined_umaps_tsv %(infile)s
    --batch_dict batch_correction/batch_dict.yml
    --cell_meta_df %(cell_mtd_file)s
    """
    # 
    # Parse the whole dict of metrics
    # this contains grouping var and qc_metrics per modality
    cmd += ' --qc_dict "%(plotqc)s"'
    cmd += " > %(outfile)s" 
    job_kwargs["job_threads"] = PARAMS['resources_threads_medium']
    P.run(cmd,**job_kwargs)


#this can follow now any mtd generation, but it will collate only RNA jobs for lisi
@follows(collate_integration_outputs)
@transform(collate_integration_outputs, 
           formatter(),  'logs/lisi.log')
def run_lisi(infile, outfile):
    cell_mtd_file = sprefix + "_cell_mtd.csv"  
    cmd = """python %(py_path)s/run_lisi.py 
    --combined_umaps_df %(infile)s 
    --cell_meta_df %(cell_mtd_file)s
    --integration_dict batch_correction/batch_dict.yml
    --fig_dir figures/  > %(outfile)s 
    """ 
    job_kwargs["job_threads"] = PARAMS['resources_threads_low']
    P.run(cmd,**job_kwargs)



@follows(run_unimodal_integration, run_multimodal_integration,run_lisi, plot_umaps)
@originate("logs/batch_correction_complete.log")
def batch_correction(outfile):
    IOTools.touch_file(outfile)





@follows( batch_correction)
def full():
    """
    All cgat pipelines should end with a full() function which updates,
    if needed, all branches of the pipeline.
    The @follows statement should ensure that all functions are covered,
    either directly or as prerequisites.
    """
    pass

# Final choices: Leave blank until you have reviewed the results from running
# sc_pipelines integration make full
# choose the parameters to integrate into the final anndata object
# then run
# `panpipes integration make merge_integration`
@originate(sprefix + "_corrected.h5mu")
def merge_integration(output_mudata):
    cmd = """
    python %(py_path)s/batch_correct_merge.py 
    --preprocessed_mudata %(preprocessed_obj)s
    --output_mudata %(output_mudata)s
    """
    choices = PARAMS['final_obj']
    if choices['rna']['include'] and choices['rna']['bc_choice'] is not None :
        rna_choice = choices['rna']['bc_choice']
        cmd += " --rna_correction_choice %(rna_choice)s"
    if choices['prot']['include'] and choices['prot']['bc_choice'] is not None:
        prot_choice = choices['prot']['bc_choice']
        cmd += " --prot_correction_choice %(prot_choice)s"
    if choices['atac']['include'] and choices['atac']['bc_choice'] is not None:
        atac_choice = choices['atac']['bc_choice']
        cmd += " --atac_correction_choice %(atac_choice)s"
    if choices['multimodal']['include'] and choices['multimodal']['bc_choice'] is not None:
        multimodal_choice = choices['multimodal']['bc_choice']
        cmd += " --multimodal_correction_choice %(multimodal_choice)s"
    cmd += " > logs/merge_final_obj.log"
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd,**job_kwargs)
    # clear up tmp since it is no longer required
    # P.run("rm -r tmp")


def main(argv=None):
    if argv is None:
        argv = sys.argv
    P.main(argv)


if __name__ == "__main__":
    sys.exit(P.main(sys.argv))



import muon as mu
import argparse
import os

import sys
import logging
L = logging.getLogger()
L.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s: %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)
L.addHandler(log_handler)


parser = argparse.ArgumentParser()
parser.add_argument('--preprocessed_mudata',
                    default='adata_scaled.h5mu',
                    help='')
parser.add_argument('--output_mudata',
                    default='mdata_corrected.h5mu',
                    help='')
parser.add_argument('--rna_correction_choice', default=None,
                    help='')
parser.add_argument('--prot_correction_choice', default=None,
                    help='')
parser.add_argument('--atac_correction_choice', default=None,
                    help='')
parser.add_argument('--multimodal_correction_choice', default=None,
                    help='')


args, opt = parser.parse_known_args()


L.info(args)

base_object = args.preprocessed_mudata

multi_mod = args.multimodal_correction_choice
if multi_mod is not None:
    base_object = "tmp/" + multi_mod.lower() + "_scaled_adata.h5mu"


    

uni_mod_paths = {}

if args.rna_correction_choice is not None:
    uni_mod_paths['rna']= "tmp/" + args.rna_correction_choice + "_scaled_adata_rna.h5ad"
if args.prot_correction_choice is not None:
    uni_mod_paths['prot']= "tmp/" + args.prot_correction_choice + "_scaled_adata_prot.h5ad"
if args.atac_correction_choice is not None:
    uni_mod_paths['atac'] = "tmp/" + args.atac_correction_choice + "_scaled_adata_atac.h5ad"
uni_mod_paths

base_obj = mu.read(base_object)


if multi_mod=="totalvi":
    totalvi_extra_obsm = {'prot': ['totalvi_protein_foreground_prob', 'totalvi_denoised_protein'], 'rna': [ 'totalvi_denoised_rna']}
    # we will want to keep these so extract them first
    totalvi_extra_obsm_keep = {}
    for k, v in totalvi_extra_obsm.items():
        print(k,v)
        totalvi_extra_obsm_keep[k] = {w:base_obj[k].obsm[w] for w in v}


# this overwrites the old modalities
for k,v in uni_mod_paths.items():
    mod_replace = mu.read(v)
    if mod_replace.shape[1] == base_obj[k].shape[1]:
        L.info('replacing the whole anndata object with batch corrected')
        base_obj.mod[k] = mod_replace
    else:
        # in this situation the vars in the mod has been subset but the 
        # original object contains all the genes.
        L.info("number of genes does not match base object, only integrating obsp and obsm into full mdata[%s] object" % k)
        base_obj.mod[k].obsm = mod_replace.obsm
        base_obj.mod[k].obsp = mod_replace.obsp
        base_obj.mod[k].uns = mod_replace.uns
        base_obj.mod[k].uns['batch_correct_gene_subset'] = mod_replace.var_names.tolist()
        # merge any extra obs columns
        new_obs = mod_replace.obs.iloc[:, ~ mod_replace.obs.columns.isin( base_obj.mod[k].obs.columns)]
        base_obj.mod[k].obs = base_obj.mod[k].obs.merge(new_obs, left_index=True, right_index=True)

if multi_mod=="totalvi":
    # restore the totalvi extras
    for k, v in totalvi_extra_obsm_keep.items():
        for w in v:
            base_obj[k].obsm[ w] = totalvi_extra_obsm_keep[k][w]

base_obj.update()
base_obj.write(args.output_mudata)

L.info("Done")





from ruffus import *
import sys
import os
from cgatcore import pipeline as P
import pandas as pd
import cgatcore.iotools as IOTools
import re
from itertools import chain, product
import glob
from panpipes.funcs.io import dictionary_stripper 

PARAMS = P.get_parameters(
    ["%s/pipeline.yml" % os.path.splitext(__file__)[0],
     "pipeline.yml"])

PARAMS['py_path'] =  os.path.join(os.path.dirname(os.path.dirname(__file__)), 'python_scripts')
PARAMS['r_path'] = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'R_scripts')

job_kwargs = {}

if PARAMS['condaenv'] is not None:
    job_kwargs["job_condaenv"] =PARAMS['condaenv']


@originate("logs/setup_dirs.sentinel")
def set_up_dirs(log_file):
    if os.path.exists("logs") is False:
        os.mkdir("logs")
    mods =  [key for key, value in PARAMS['modalities'].items() if value is True]
    for mod in mods:
        if os.path.exists(mod) is False:
            os.makedirs(os.path.join(mod))
    IOTools.touch_file(log_file)
    pass

#--------------------------------------------------
# Plot markers
#--------------------------------------------------



marker_files = PARAMS['custom_markers']['files']
marker_files  = list(chain(*[marker_files[x]  for x in ['full', 'minimal'] if marker_files[x]  is not None]))


do_plot_features = (PARAMS['custom_markers']['files']['full'] is not None or PARAMS['custom_markers']['files']['minimal'] is not None) and \
                (PARAMS['do_plots_marker_dotplots'] or PARAMS['do_plots_marker_matrixplots'])


@follows(set_up_dirs)
@active_if(do_plot_features)
@transform(marker_files, formatter(), "logs/markers_by_group__{basename[0]}.log")
def plot_custom_markers_per_group(marker_file, log_file):
    print(marker_file)
    print(log_file)
    group_vars = " ".join(PARAMS["grouping_vars"])
    # pull out the modalities to use, not including multimodal.
    modalities = [key for key, val in PARAMS['modalities'].items() if val and key not in ["multimodal", "rep"]]
    modalities = ",".join(modalities)
    # pull out the (optional) layer choices
    layer_vars = {key:val  for key, val in PARAMS['custom_markers_layers'].items() if PARAMS['modalities'][key]}
    layer_vars = dictionary_stripper(layer_vars)
    cmd = """
        python %(py_path)s/plot_custom_markers.py \
            --infile %(mudata_obj)s \
            --modalities %(modalities)s \
            --layers "%(layer_vars)s" \
            --marker_file %(marker_file)s \
            --group_cols %(group_vars)s \
            --base_figure_dir ./
     """
    cmd += " > %(log_file)s "
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)


plot_embeddings = any([True if val['run'] is True else False for val in PARAMS["embedding"].values() ])
@active_if(plot_embeddings )
@follows(set_up_dirs)
@active_if(PARAMS['custom_markers']['files']['minimal'] is not None)
@transform(PARAMS['custom_markers']['files']['minimal'], formatter(), "logs/markers_umap_{basename[0]}.log")
def plot_custom_markers_umap(marker_file, log_file):
    embedding_dict = PARAMS["embedding"]
    embedding_dict =  {mod:val['basis'] for mod, val in embedding_dict.items() if val['run'] is True}
    embedding_dict = dictionary_stripper(embedding_dict)
    layer_vars = {key:val  for key, val in PARAMS['custom_markers_layers'].items() if PARAMS['modalities'][key]}
    layer_vars = dictionary_stripper(layer_vars)
    modalities = ",".join([key for key, val in PARAMS['modalities'].items() if val])
    cmd = """
        python %(py_path)s/plot_custom_markers_umap.py \
            --infile %(mudata_obj)s \
            --modalities %(modalities)s \
            --layers "%(layer_vars)s" \
            --basis_dict "%(embedding_dict)s" \
            --marker_file %(marker_file)s \
            --base_figure_dir ./
    """
    cmd += " > %(log_file)s "
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)




# @transform(PARAMS['custom_markers']['files']['minimal'], formatter(), "logs/markers_umap__{basename[0]}.log")
@active_if(plot_embeddings )
@follows(set_up_dirs)
@originate("logs/categorical_variables_umap.log")
def plot_categorical_umaps(log_file):
    embedding_dict = PARAMS["embedding"]
    embedding_dict =  {mod:val['basis'] for mod, val in embedding_dict.items() if val['run'] is True}
    cat_vars = PARAMS['categorical_vars']
    if "all" in cat_vars.keys() and cat_vars["all"] is not None:    
        for k, v in cat_vars.items():
            if v is None:
                cat_vars[k] = cat_vars['all']
            else:
                cat_vars[k] = cat_vars[k] + cat_vars['all']
    del cat_vars['all']
    # subset by the modalities we are using according to PARAMS['modalities']
    cat_vars = {key:val  for key, val in cat_vars.items() if PARAMS['modalities'][key] }
    layer_vars = {key:val  for key, val in PARAMS['custom_markers_layers'].items() if PARAMS['modalities'][key]}
    cmd = """
        python %(py_path)s/plot_variables_umaps.py \
            --infile %(mudata_obj)s \
            --layers "%(layer_vars)s" \
            --basis_dict "%(embedding_dict)s" \
            --categorical_variables "%(cat_vars)s" \
            --base_figure_dir ./
            --fig_suffix categorical_vars.png
    """
    cmd += " > %(log_file)s "
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)


@active_if(plot_embeddings)
@follows(set_up_dirs)
@originate("logs/continuous_variables_umap.log")
def plot_continuous_umaps(log_file):
    embedding_dict = PARAMS["embedding"]
    embedding_dict =  {mod:val['basis'] for mod, val in embedding_dict.items() if val['run'] is True}
    cont_vars = PARAMS['continuous_vars']
    if "all" in cont_vars.keys() and cont_vars["all"] is not None:    
        for k, v in cont_vars.items():
            if v is None:
                cont_vars[k] = cont_vars['all']
            else:
                cont_vars[k] = cont_vars[k] + cont_vars['all']
    del cont_vars['all']
    # subset by the modalities we are using according to PARAMS['modalities']
    cont_vars = {key:val  for key, val in cont_vars.items() if PARAMS['modalities'][key] }
    layer_vars = {key:val  for key, val in PARAMS['custom_markers_layers'].items() if PARAMS['modalities'][key]}
  
    cmd = """
        python %(py_path)s/plot_variables_umaps.py 
            --infile %(mudata_obj)s \
            --layers "%(layer_vars)s" 
            --basis_dict "%(embedding_dict)s" 
            --continuous_variables "%(cont_vars)s" 
            --base_figure_dir ./
            --fig_suffix continuous_vars.png
    """
    cmd += " > %(log_file)s "
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)


do_plot_metrics = any([PARAMS['do_plots_categorical_barplots'],
                PARAMS['do_plots_categorical_stacked_barplots'],
                PARAMS['do_plots_continuous_violin']])
# write metadata out
@active_if(do_plot_metrics)
@originate(PARAMS['sample_prefix'] + "_cell_metadata.tsv")
def write_obs(cmtd):
    cmd = """
        python %(py_path)s/write_metadata.py
        --infile %(mudata_obj)s
        --outfile %(cmtd)s > logs/write_obs.log
        """
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)
  

# Plot cluster metrics
@follows(set_up_dirs)
@active_if(do_plot_metrics)
@transform(write_obs, formatter(), "logs/plot_metrics.log")
def plot_metrics(mtd, log_file):
    cmd = """
    Rscript %(r_path)s/plot_metrics.R \
        --mtd_object %(mtd)s \
        --params_yaml pipeline.yml > %(log_file)s
    """
    job_kwargs["job_threads"] = PARAMS['resources_threads_low']
    P.run(cmd, **job_kwargs)


# parse the scatter files
scatter_files = [PARAMS['paired_scatters'], PARAMS['custom_markers_files']['paired_scatters']]
scatter_files = [sc if type(sc) is list else [sc] for sc in scatter_files if sc is not None]
scatter_files = list(set(chain(*scatter_files)))
@follows(set_up_dirs)
@active_if(PARAMS['do_plots_paired_scatters'])
@active_if(len(scatter_files) != 0)
@transform(scatter_files,
            formatter(), "logs/scatters__{basename[0]}.log")
def plot_scatters(infile, log_file):
    print(infile)
    cmd = """
        python %(py_path)s/plot_features_scatter.py
        --mdata_object %(mudata_obj)s
        --layers_dict "%(custom_markers_layers)s" 
        --scatters_csv %(infile)s
        > %(log_file)s
        """
    job_kwargs["job_threads"] = PARAMS['resources_threads_high']
    P.run(cmd, **job_kwargs)    


@follows(plot_custom_markers_per_group, plot_custom_markers_umap,
 plot_continuous_umaps, plot_categorical_umaps, plot_metrics, plot_scatters)
def full():
    """
    All cgat pipelines should end with a full() function which updates,
    if needed, all branches of the pipeline.
    The @follows statement should ensure that all functions are covered,
    either directly or as prerequisites.
    """
    pass


def main(argv=None):
    if argv is None:
        argv = sys.argv
    P.main(argv)


if __name__ == "__main__":
    sys.exit(P.main(sys.argv))



"""
Scanpy inbuilt plots for metadata variables
CRG 2020-06-19
"""

import scanpy as sc
import muon as mu
from anndata import AnnData
import pandas as pd
import argparse
from panpipes.funcs.io import read_yaml, dictionary_stripper
from itertools import product, chain

sc.settings.autoshow = False
import os
import matplotlib
matplotlib.use('agg')

import sys
import logging
L = logging.getLogger()
L.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s: %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)
L.addHandler(log_handler)

# parse argumetns
parser = argparse.ArgumentParser()
parser.add_argument("--infile",
                    default="pbmc3k.h5mu",
                    help="file name, format: .h5mu")
parser.add_argument("--basis_dict",
                    default="X_umap",
                    help="basis in obsm to plot")
parser.add_argument("--categorical_variables",
                    default=None,
                    help="categorical values within mdata.obs")
parser.add_argument("--continuous_variables",
                    default=None,
                    help="continuous values within mdata.obs")
parser.add_argument("--base_figure_dir", default="figures/",
                    help="figures path")
parser.add_argument("--fig_suffix", default="variables.png",
                    help="figures path")
args, opt = parser.parse_known_args()

L.info("running with:")
L.info(args)
# args = argparse.Namespace(infile='../run_clustering_mm/mdata_clustered.h5mu',
#  basis_dict="{'prot': ['X_umap', 'X_pca']}", 
#  base_figure_dir='./')

# ---------
def main(adata, mod, plot_features,  basis, fig_suffix):
    # define file name
    fname_prefix = "_".join(["_" + mod,  fig_suffix])
    # get features
    L.info("plotting")
    L.info(plot_features)
    sc.settings.figdir  = os.path.join(args.base_figure_dir, mod)
    plot_features = [pf for pf in plot_features if pf in mdata.obs.columns]
    pointsize = 120000 / adata.shape[0]
    mu.pl.embedding(adata, basis=basis, color=plot_features, size=pointsize, save = fname_prefix + ".png")


L.debug("load data")
mdata = mu.read(args.infile)


# get bases
basis_dict = dictionary_stripper(read_yaml(args.basis_dict))

if args.categorical_variables is not None:
    cat_vars = dictionary_stripper(read_yaml(args.categorical_variables))
    
    uniq_discrete = list(set(chain(*cat_vars.values())))
    # make sure they are categories
    mdata.obs[uniq_discrete] = mdata.obs[uniq_discrete].apply(lambda x: x. astype('category'))
else:
    cat_vars = {}


if args.continuous_variables is not None:
    try:
        cont_vars = dictionary_stripper(read_yaml(args.continuous_variables))
    except AttributeError:
        # this assumes that we have tried to parse a dict and nstead found a string
        # there is probably a better solution
        cont_vars = args.continuous_variable_dict
else:
    cont_vars = {}




# we have multimodal object
params_dict = {}
for mod in ['rna', 'prot', 'atac', 'multimodal']:
    params_dict['mod' ] = mod
    try:
        basis_list = basis_dict[mod]
    except KeyError:
        L.info("no basis given for mod: %s, no plot's produced" % mod)
        continue
    params_dict['plot_features'] = []
    if mod in cat_vars.keys():
        params_dict['plot_features'] = cat_vars[mod]
    if mod in cont_vars.keys():
        params_dict['plot_features'] = params_dict['plot_features'] +  cont_vars[mod]
    params_dict['fig_suffix'] = args.fig_suffix
    L.info("plotting with params")
    L.info(params_dict)
    if len(basis_list) > 0:
        for basis in basis_list:
            main(adata=mdata, basis = mod + ":" + basis, **params_dict)



L.info("Done")





import scanpy as sc
import argparse
import pandas as pd
import os


from panpipes.funcs.processing import test_file_or_value, merge_with_adata_obs, check_for_bool
from panpipes.funcs.io import read_anndata, write_anndata 

import sys
import logging
L = logging.getLogger()
L.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s: %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)
L.addHandler(log_handler)

sc.settings.verbosity = 0  # verbosity: errors (0), warnings (1), info (2), hints (3)
# comment this because of numba issues
# sc.logging.print_versions()

# parse arguments
parser = argparse.ArgumentParser()

parser.add_argument('--input_anndata',
                    default='data/anndata-n5000-filt.h5ad',
                    help='')
parser.add_argument('--output_anndata',
                    default='data/anndata-n5000-filt.h5ad',
                    help='')
parser.add_argument('--use_muon',
                    default=False,type=check_for_bool,
                    help='')
parser.add_argument('--output_prefix',
                    default='',
                    help='prefix to prepend to saved files. here just used for metadata file')
parser.add_argument('--scrublet_cutoff', default=None,
                    help='this is either a value or a file')
parser.add_argument('--batch_col', default="sample_id",
                    help='')

L.warning("Running subtract scrublet")

parser.set_defaults(verbose=True)
args, opt = parser.parse_known_args()

use_muon = args.use_muon

# load data
# if muon is True then rna only is loaded
# if muon is False then the modality argument is ignored and the normal anndata h5ad format is loaded
adata = read_anndata(args.input_anndata, use_muon=use_muon, modality="rna")

# test what kind of input we are dealing with, value or file?
run = test_file_or_value(args.scrublet_cutoff)


# adata=sc.read_h5ad("./data/run_integration/taurus_test_filt.h5ad")

# run = test_file_or_value("./data/run_integration/scrublet_cutoffs.csv")
# subtract scrublet
if run == 'value':
    # if args.scrublet_cutoff is not None:
    # if it is a value
    adata = adata[adata.obs['doublet_scores'] < float(args.scrublet_cutoff), :]
elif run == 'file':
    # if it is a file
    scr_df = pd.read_csv(args.scrublet_cutoff, names=["sample_id", "scrublet_cutoff"])
    adata.obs = merge_with_adata_obs(adata.obs, scr_df, on_col="sample_id")
    adata = adata[adata.obs['doublet_scores'] < adata.obs['scrublet_cutoff'],]
else:
    pass

L.info("saving anndata and obs in a metadata tsv file")
metafile = adata.obs
metafile["cellbarcode"] = adata.obs.index

savename= args.output_prefix + "_filtered_cell_metadata.tsv"
metafile.to_csv(savename, sep='\t', index=True)  

write_anndata(adata, args.input_anndata, use_muon=use_muon, modality="rna")

L.info("done")


import argparse
import os
import sys
import logging
import scanpy as sc
from muon import MuData, read
from panpipes.funcs.scmethods import run_neighbors_method_choice

L = logging.getLogger()
L.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s: %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)
L.addHandler(log_handler)


parser = argparse.ArgumentParser()
parser.add_argument('--infile', default='./anndata.h5mu',
                    help="file name, format: anndata_scaled.h5ad")
parser.add_argument('--modality',
                    default="rna",
                    help='')
parser.add_argument('--outfile', default='./anndata_wneighbors.h5ad',
                    help="file name, format: .h5ad")
parser.add_argument('--integration_method', default=None,
                    help="helps find the correct dimension reduction for sc.pp.neighbors(), default='X_pca'")
parser.add_argument('--n_threads', default=1,
                    help="num threads to use for neighbor computations")
parser.add_argument('--neighbors_n_pcs',
                    help="n_pcs")
parser.add_argument('--neighbors_method',
                    help="neighbours method, scanpy or hnsw")
parser.add_argument('--neighbors_k',
                    help="neighbors k")
parser.add_argument('--neighbors_metric',
                    help="neighbor metric, e.g. euclidean or cosine")
args, opt = parser.parse_known_args()


sc.settings.n_jobs = int(args.n_threads)
L.info("Running with options: %s", args)

# read data
L.info("reading adata")
mdata = read(args.infile)
if type(mdata) is MuData:
    adata=mdata[args.modality]

if "X_pca" not in adata.obsm.keys():
    sc.tl.pca(adata)

# run command
opts = dict(method=args.neighbors_method,
            n_neighbors=int(args.neighbors_k),
            n_pcs=int(args.neighbors_n_pcs),
            metric=args.neighbors_metric,
            nthreads=args.n_threads)

dim_red_location = {
    "None": "X_pca",
    "combat": "X_pca",
    "scanorama": "X_scanorama",
    "harmony" : "X_harmony" ,
    "scvi": "X_scVI",
    "totalvi" : "X_totalVI",

}
opts['use_rep'] = dim_red_location[str(args.integration_method)]

if args.integration_method == 'bbknn':
    L.info("bbknn neighbours should already be included in the object, not rerunning.")
    try:
        adata.uns['neighbors']
    except KeyError:
        sys.exit("bbknn neighbours not found, make sure they are in the scaled object")
    pass
else:
    run_neighbors_method_choice(adata,
                                **opts)



L.info("saving data")
if os.path.exists(os.path.basename(args.outfile)) is False:
    os.makedirs(os.path.basename(args.outfile))
adata.write(args.outfile)


L.info("Done")




from venv import create
from ruffus import *
import sys
import os
from cgatcore import pipeline as P
import pandas as pd
import cgatcore.iotools as IOTools
import re
from itertools import chain
import glob

# __file__="/well/cartography/users/zsj686/non_cart_projects/005-multimodal_scpipelines/src/sc_pipelines_muon_dev/panpipes/pipeline_refmap.py"
PARAMS = P.get_parameters(
    ["%s/pipeline.yml" % os.path.splitext(__file__)[0],
     "pipeline.yml"])

job_kwargs = {}

if PARAMS['condaenv'] is not None:
    job_kwargs["job_condaenv"] =PARAMS['condaenv']


PARAMS['py_path'] =  os.path.join(os.path.dirname(os.path.dirname(__file__)), 'python_scripts')
PARAMS['r_path'] = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'R_scripts')
PARAMS['resources_path'] = os.path.join(os.path.dirname(os.path.dirname(__file__)), "resources")



if PARAMS['totalvi'] is not None:
    totalvi=True
    totalvi_models = PARAMS['totalvi']

if PARAMS['scvi'] is not None:
    if PARAMS['scanvi'] is not None:
        scvi=True


@originate("logs/setup_dirs.sentinel")
def set_up_dirs(log_file):
    os.makedirs("models",exist_ok=True)
    os.makedirs("refmap",exist_ok=True)
    os.makedirs("logs",exist_ok=True)
    os.makedirs("figures",exist_ok=True)
    IOTools.touch_file(log_file)
    pass

# ------------------------------
# create cluster job for each model in scvi tools: scvi, scanvi, totalvi
# ------------------------------

def gen_refmap_jobs():
    for ref_architecture in ["scvi", "scanvi", "totalvi"]:
        if PARAMS[ref_architecture] is not None:
            mod_list = PARAMS[ref_architecture]
            if type(mod_list) is not list:
                mod_list = [mod_list]
            for infile in mod_list:
                # this is creating output files based on the model folder name
                out_prefix=os.path.basename(os.path.dirname(infile))
                model_name = ''.join(e for e in out_prefix if e.isalnum())
                latent_choice = "X_" + ref_architecture
                file_name= "umap_" + model_name + "_" + latent_choice
                outfile = os.path.join("./refmap/",(file_name + ".csv"))

                log_file = os.path.join('logs/', out_prefix + ".log")
                yield infile, outfile, log_file, ref_architecture
                


# error if modality is not rna

# ------------------------------
# refmap scvi tools: scvi, scanvi, totalvi
# ------------------------------

@follows(set_up_dirs)
@files(gen_refmap_jobs)
def run_refmap_scvi(infile, outfile, log_file, ref_architecture ):
    
    cmd = """
    python %(py_path)s/refmap_scvitools.py
    --query_data %(query)s
    --reference_path %(infile)s
    --reference_architecture  %(ref_architecture)s
    --neighbors_n_pcs %(neighbors_npcs)s
    --neighbors_method %(neighbors_method)s
    --neighbors_k %(neighbors_k)s
    --neighbors_metric %(neighbors_metric)s
    --outfile %(outfile)s
    """
    if PARAMS['query_celltype'] is not None:
        cmd += " --query_celltype %(query_celltype)s"
    if PARAMS['query_batch'] is None:
        if ref_architecture=="totalvi":
            if PARAMS['transform_batch'] is not None:
                cmd += " --transform_batch %(transform_batch)s"
    else:
        cmd += " --transform_batch %(query_batch)s"    #what do we need this for in the Q2R rna
    if PARAMS['reference_data'] is not None:
        cmd += " --adata_reference %(reference_data)s "
    if ref_architecture == "totalvi":
       cmd += "  --impute_proteins %(impute_proteins)s"
    if PARAMS['run_randomforest'] is not None:
        cmd += " --predict_rf %(run_randomforest)s"
       
    cmd += " > %(log_file)s"
    job_kwargs["job_threads"] = PARAMS['resources_threads_low']
    P.run(cmd, **job_kwargs)


##TODO remove this collate job cause plotting is now in the script
# @collate([run_refmap_scvi], #multimodal
#          regex(r"refmap/(.*)"), # this does nothing
#           r'refmap/combined_umaps.tsv')
# def plot_umaps(infiles,outfile):
#     infiles_string = ','.join(infiles)
#     cell_mtd_file = "cell_mtd.csv"
#     cmd = """python %(py_path)s/refmap_plot_umaps.py 
#     --input_mudata %(query)s
#     --input_umap_files %(infiles_string)s 
#     --output_combined_umaps_tsv %(outfile)s
#     """ 
#     cmd += " > logs/collate_outputs.log "
#     job_kwargs["job_threads"] = PARAMS['resources_threads_low']
#     P.run(cmd, **job_kwargs)  
@active_if(PARAMS["scib_run"])
@transform(run_refmap_scvi,
           regex(r"refmap/umap_(.*).csv"),
           r'logs/refmapscib_\1.log')
def run_scib_refmap(infile,logfile):
    str_use = os.path.splitext(os.path.basename(infile).replace("umap_", "").replace(".csv", ""))[0]
    mudata_input= "query_to_reference_" + str_use + ".h5mu"
    repuse = "X_"+ str_use.split("_")[-1]    
    cmd="""
    python %(py_path)s/refmap_scib.py
    --query ./refmap/%(mudata_input)s
    --repuse %(repuse)s
    --outdir ./refmap/
    """
    if PARAMS["scib_batch_key"] is not None:
        cmd += " --batch_key %(scib_batch_key)s"
    else: 
        if PARAMS["query_batch"] is not None:
            cmd += " --batch_key %(query_batch)s" 
    if PARAMS["scib_cluster_key"] is not None:
        cmd += " --cluster_key %(scib_cluster_key)s" 
    if PARAMS["scib_celltype_key"] is not None:
        cmd += " --covariate %(scib_celltype_key)s"
    else: 
        if PARAMS['query_celltype'] is not None:
            cmd += " --covariate %(query_celltype)s"
    cmd += " > %(logfile)s"
    job_kwargs["job_threads"] = PARAMS['resources_threads_medium']
    P.run(cmd, **job_kwargs)  


@follows(run_scib_refmap)
def full():
    """
    All cgat pipelines should end with a full() function which updates,
    if needed, all branches of the pipeline.
    The @follows statement should ensure that all functions are covered,
    either directly or as prerequisites.
    """
    #IOTools.touch_file(file)
    pass

def main(argv=None):
    if argv is None:
        argv = sys.argv
    P.main(argv)


if __name__ == "__main__":
    sys.exit(P.main(sys.argv))



"""
Fetching data for plotting
CRG 2020-06-19
"""

import scanpy as sc
import muon as mu
from anndata import AnnData

import pandas as pd
import argparse
from itertools import product
sc.settings.autoshow = False
import os
import matplotlib
matplotlib.use('agg')
import re
from panpipes.funcs.io import  read_yaml

import sys
import logging
L = logging.getLogger()
L.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s: %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)
L.addHandler(log_handler)

# parse argumetns
parser = argparse.ArgumentParser()
parser.add_argument("--infile",
                    default="pbmc3k.h5ad",
                    help="file name, format: .h5ad")
parser.add_argument("--modalities",
                    default="rna",
                    help="modality keys containing data to be plotted, comma spearated list e.g. rna,prot default 'rna'")
parser.add_argument("--layers",
                    default="X",
                    help="layer containing data to be plotted, default=X")
parser.add_argument("--group_cols", default=["sample_id"],
                help="grouping variables", nargs='+',)
parser.add_argument("--marker_file",
                    default="Marker_Lists-myeloid.csv",
                    help="comma separateed list of marker_list csvs, one column of gene_ids")
parser.add_argument("--base_figure_dir", default="figures/",
                    help="figures path")


args, opt = parser.parse_known_args()
L.info("running with:")
L.info(args)
sc.settings.figdir = args.base_figure_dir

# ---- script

def main(adata, mod, df, grouping_var, pfx, layer_choice=None):
    for gc in df['group'].unique():
        # define file name
        if layer_choice is None or layer_choice =="X":
            layer_choice = None
            layer_string = ""
        else:
            layer_string = layer_choice
        # get features
        fetches = df[df['group'] == gc]['feature']
        plot_features = [gg for gg in fetches if gg in adata.var_names]
        plot_features = list(set(plot_features))
        # do PCA if it is missing, (prerequisite for dendrogram)
        if "X_pca" not in adata.obsm.keys():
            sc.pp.pca(adata)
        use_dendrogram=True
        if len(adata.obs[grouping_var].unique()) > 2:
            try:
                sc.tl.dendrogram(adata, grouping_var, use_rep="X_pca",linkage_method="average")
                use_dendrogram=True
            except ValueError: 
                use_dendrogram=False

        sc.settings.figdir  = os.path.join(args.base_figure_dir, mod ,  re.sub(":", "_", grouping_var))
        fname_prefix = "_".join([layer_string, pfx, gc ])
        fname_prefix = re.sub(":", "_", fname_prefix)
        sc.pl.dotplot(adata,
                        var_names=plot_features,
                        groupby=grouping_var,
                        layer=layer_choice,
                        dendrogram=use_dendrogram,
                        save=fname_prefix + '.png',
                        figsize=(24, 5))
        sc.pl.matrixplot(adata,
                        var_names=plot_features,
                        groupby=grouping_var,
                        dendrogram=use_dendrogram,
                        layer=layer_choice,
                        save=fname_prefix + '.png',
                        figsize=(24, 5))

L.debug("load data")
mdata = mu.read(args.infile)
modalities = args.modalities.split(',')

L.debug("load marker file")
df = pd.read_csv(args.marker_file )

pfx = re.sub(".csv", "csv", os.path.basename(args.marker_file))
# write out the features that are not found in adata var (due to filtering, or incorrect name)
not_found = [gg for gg in df['feature'] if gg not in mdata.var_names]
not_found_file = re.sub(".csv", "_features_not_found.txt", os.path.basename(args.marker_file))
if len(not_found) > 0 :
    with open(not_found_file, 'w') as f:
        for item in not_found:
            f.write("%s\n" % item)
            
# get layer
try:
    layers = read_yaml(args.layers)
except AttributeError:
    # this assumes that we have tried to parse a dict and nstead found a string
    # there is probably a better solution
    layers = args.layers

group_vars = args.group_cols

if type(mdata) is AnnData:
    adata = mdata
    for gv in group_vars:
        adata.obs[gv] = adata.obs[gv].astype('category')
        main(adata, layer_choice=args.layers, group = gv, pfx = pfx, df = df)
else:
    # we have multimodal object
    for mod in modalities:
        print(mod)
        df_sub = df[df['mod'] == mod]
        for gv in group_vars:
            mdata[mod].obs[gv] = mdata.obs.loc[mdata[mod].obs_names,gv].astype('category')
        mdata.update_obs()
        try:
            ll = layers[mod]
        except KeyError:
            ll = [None]
        if len(group_vars) > 0 and ll is not None:
            for gv, layer in product(group_vars, ll):
                print(gv, layer)
                main(adata=mdata[mod], 
                    mod=mod,
                    layer_choice = layer,
                    grouping_var=gv, 
                    pfx = pfx,
                    df=df_sub)

L.info("Done")




'''
Preprocess spatial transcriptomics data
'''
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)

import numpy as np
import scanpy as sc
import muon as mu
import scanpy.experimental as sce

import os
import argparse
import sys
import logging
import re

L = logging.getLogger()
L.setLevel(logging.INFO)
log_handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s: %(levelname)s - %(message)s')
log_handler.setFormatter(formatter)
L.addHandler(log_handler)
L.debug("testing logger works")


from panpipes.funcs.scmethods import X_is_raw


sc.settings.verbosity = 3

parser = argparse.ArgumentParser()

parser.add_argument("--input_mudata",
                    default="mudata_unfilt.h5mu",
                    help="")
parser.add_argument("--output_mudata",
                    default="mudata_unfilt.h5mu",
                    help="")
parser.add_argument("--figdir",
                    default="./figures/",
                    help="path to save the figures to")

parser.add_argument("--norm_hvg_flavour",
                    default=None,
                    help="how to normalize the data and perform HVG selection, either 'squidpy' or 'seurat'")
parser.add_argument("--n_top_genes",
                    default=2000,
                    help="how many variable genes to select")
parser.add_argument("--filter_by_hvg",
                    default=False,
                    help="if True, subset the data to highly-variable genes after finding them")
parser.add_argument("--hvg_batch_key",
                    default=None,
                    help="if specified, highly-variable genes are selected within each batch separately and merged")
parser.add_argument("--squidpy_hvg_flavour",
                    default='seurat',
                    help="flavor for identifying highly variable genes. 'seurat', 'cellranger' or 'seurat_v3'")
parser.add_argument("--min_mean",
                    default=0.05,
                    help="min mean expression for HVG selection")
parser.add_argument("--max_mean",
                    default=1.5,
                    help="max mean expression for HVG selection")
parser.add_argument("--min_disp",
                    default=0.5,
                    help="dispersion min for HVG selection")
parser.add_argument("--theta",
                    default=100,
                    help="the negative binomial overdispersion parameter theta for Pearson residuals")
parser.add_argument("--clip",
                    default=None,
                    help="determines if and how residuals are clipped. Further information: https://scanpy.readthedocs.io/en/stable/generated/scanpy.experimental.pp.normalize_pearson_residuals.html")
parser.add_argument("--n_pcs",
                    default=50,
                    help="how many PCs to compute")


args, opt = parser.parse_known_args()

L.info("running with args:")
L.info(args)

figdir = args.figdir

if not os.path.exists(figdir):
    os.mkdir(figdir)

sc.settings.figdir = figdir
sc.set_figure_params(scanpy=True, fontsize=14, dpi=300, facecolor='white', figsize=(5,5))

mdata = mu.read(args.input_mudata)
spatial = mdata.mod['spatial']

input_data = os.path.basename(args.input_mudata)
pattern = r"_filtered.h5(.*)"
match = re.search(pattern, input_data)
sprefix = input_data[:match.start()]


# check if raw data is available
#maybe layer of raw data as parameter
if X_is_raw(spatial):
    spatial.layers['raw_counts'] = spatial.X.copy()
elif "raw_counts" in spatial.layers :
    L.info("raw_counts layer already exists")
    spatial.X = spatial.layers['raw_counts'].copy()
else:
    L.error("X is not raw data and 'raw_counts' layer not found")
    sys.exit("X is not raw data and 'raw_counts' layer not found")



# Normalization + HVG selection based on flavour
if args.norm_hvg_flavour == "squidpy":
    if args.squidpy_hvg_flavour == "seurat_v3":
        sc.pp.highly_variable_genes(spatial, flavor="seurat_v3", n_top_genes=int(args.n_top_genes), subset=args.filter_by_hvg,
                                    batch_key=args.hvg_batch_key)
        sc.pp.normalize_total(spatial)
        sc.pp.log1p(spatial)
    else:
        sc.pp.normalize_total(spatial)
        sc.pp.log1p(spatial)
        sc.pp.highly_variable_genes(spatial, flavor=args.squidpy_hvg_flavour,
                                    min_mean=float(args.min_mean),
                                    max_mean=float(args.max_mean),
                                    min_disp=float(args.min_disp), subset=args.filter_by_hvg, batch_key=args.hvg_batch_key)
    spatial.layers["lognorm"] = spatial.X.copy()
    # plot HVGs:
    sc.pl.highly_variable_genes(spatial, show=False, save="_genes_highlyvar" + "."+ sprefix+ ".png")

elif args.norm_hvg_flavour == "seurat":
    if args.clip is None:
        clip = args.clip
    elif args.clip == "None":
        clip = None
    elif args.clip == "np.Inf":
        clip = np.Inf
    else:
        clip = float(args.clip)

    sce.pp.highly_variable_genes(spatial, theta=float(args.theta), clip=clip, n_top_genes=int(args.n_top_genes),
                                 batch_key=args.hvg_batch_key, flavor='pearson_residuals',
                                 layer="raw_counts", subset=args.filter_by_hvg)
    
    sce.pp.normalize_pearson_residuals(spatial, theta=float(args.theta), clip=clip, layer="raw_counts")
    spatial.layers["norm_pearson_resid"] = spatial.X.copy()
else:
    # error or warning?
    L.warning("No normalization and HVG selection was performed! To perform, please specify the 'norm_hvg_flavour' as either 'squidpy' or 'seurat'")


if "highly_variable" in spatial.var:
    L.warning( "You have %s Highly Variable Features", np.sum(spatial.var.highly_variable))



#PCA
sc.pp.pca(spatial, n_comps=int(args.n_pcs), svd_solver='arpack', random_state=0)
sc.pl.pca(spatial, save = "_vars" + "."+ sprefix+".png")
sc.pl.pca_variance_ratio(spatial, log=True, n_pcs=int(args.n_pcs), save= "."+ sprefix+".png")


        
mdata.update()
mdata.write(args.output_mudata)

L.info("Done")

